package com.gic.evaluate.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.evaluate.dto.EvaluateDTO;
import com.gic.evaluate.dto.EvaluateQuickDTO;
import com.gic.evaluate.dto.UserReplyDTO;
import com.gic.evaluate.qo.EvaluateQO;

import java.util.List;

/**
 * 后台评价管理服务
 * @Description:
 * @author zhiwj
 * @date 2019-12-26 17:54
 */
public interface EvaluateManageApiService {

    /**
     * 后台页面的查询
     * @Title: listEvaluate
     * @Description: 
     * @author zhiwj 
     * @param evaluateQO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.evaluate.dto.EvaluateDTO>
     * @throws 
     */
    ServiceResponse<Page<EvaluateDTO>> listEvaluateForManage(EvaluateQO evaluateQO);

    /**
     * 查评价详情
     * @Title: getEvaluate
     * @Description:
     * @author zhiwj
     * @param evaluateId
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.evaluate.dto.EvaluateDTO>
     * @throws
     */
    ServiceResponse<EvaluateDTO> getEvaluate(Integer enterpriseId, Integer evaluateId);

    /**
     * 修改展示状态
     * @Title: updateShowStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param evaluateId
     * @param showStatus
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> updateShowStatus(Integer enterpriseId, List<Integer> evaluateId, Integer showStatus);
    
    /**
     * 编辑回复是否展示状态
     * @Title: updateReplyShowStatus
     * @Description: 
     * @author zhiwj 
     * @param enterpriseId
     * @param evaluateReplyId
     * @param showStatus
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws 
     */
    ServiceResponse<Void> updateReplyShowStatus(Integer enterpriseId, Integer evaluateReplyId, Integer showStatus);

    /**
     * 修改是否为热评置顶
     * @Title: updateTopHotStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param evaluateId
     * @param topHotStatus
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> updateTopHotStatus(Integer enterpriseId, Integer evaluateId, Integer topHotStatus);

    /**
     * 修改是否为恶评
     * @Title: updateMaliciousStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param evaluateId
     * @param maliciousStatus
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> updateMaliciousStatus(Integer enterpriseId, Integer evaluateId, Integer maliciousStatus);

    /**
     * 管理员回复
     * @Title: replyEvaluate
     * @Description:
     * @author zhiwj
     * @param userReplyDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> replyEvaluate(UserReplyDTO userReplyDTO);

    /**
     * 编辑回复
     * @Title: editReplyEvaluate
     * @Description:
     * @author zhiwj
     * @param userReplyDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> editReplyEvaluate(UserReplyDTO userReplyDTO);

    /**
     * 删除回复
     * @Title: delReply
     * @Description:
     * @author zhiwj
     * @param evaluateReplyId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> delReply(Integer evaluateReplyId);

    /**
     * 快捷标签表
     * @Title: listEvaluateQuickReply
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param quickReplyType
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.evaluate.dto.EvaluateQuickDTO>
     * @throws
     */
    ServiceResponse<List<EvaluateQuickDTO>> listEvaluateQuickReply(Integer enterpriseId, Integer quickReplyType);

    /**
     * 按个数求
     * @Title: countEvaluate
     * @Description:
     * @author zhiwj
     * @param evaluateQO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> countEvaluate(EvaluateQO evaluateQO);

    /**
     * 计算门店分数
     * @Title: computeStoreScore
     * @Description: 
     * @author zhiwj
     * @param msg
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws 
     */
    ServiceResponse<Integer> computeStoreScoreEvaluate(String msg);

    /**
     * 重新计算某一个门店的分数
     * @Title: computeStoreScoreEvaluate
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> computeStoreScoreEvaluate(Integer enterpriseId, Integer storeId);
}
