package com.gic.evaluate.dao.mapper;

import com.gic.evaluate.entity.TabEvaluate;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.qo.StoreScoreQO;
import com.gic.evaluate.qo.StoreTrendQO;
import com.github.pagehelper.Page;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 
 * @ClassName: TabEvaluateMapper
 * @Description:  
 * @author zhiwj
 * @date 2020-09-08 18:50
 */
public interface TabEvaluateMapper {
    /**
     * 根据主键删除
     *
     * @param evaluateId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer evaluateId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabEvaluate record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabEvaluate record);

    /**
     * 根据主键查询
     *
     * @param evaluateId 主键
     * @return 实体对象
     */
    TabEvaluate selectByPrimaryKey(Integer evaluateId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabEvaluate record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabEvaluate record);

    /**
     * listEvaluate
     * @Title: listEvaluate
     * @Description:
     * @author zhiwj
     * @param evaluateQO
     * @return com.github.pagehelper.Page<com.gic.evaluate.entity.TabEvaluate>
     * @throws
     */
    Page<TabEvaluate> listEvaluate(EvaluateQO evaluateQO);

    /**
     * countEvaluate
     * @Title: countEvaluate
     * @Description:
     * @author zhiwj
     * @param evaluateQO
     * @return java.lang.Integer
     * @throws
     */
    Integer countEvaluate(EvaluateQO evaluateQO);

    /**
     * updateShowStatus
     * @Title: updateShowStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param evaluateId
     * @param showStatus
     * @return void
     * @throws
     */
    void updateShowStatus(@Param("enterpriseId") Integer enterpriseId, @Param("evaluateId") List<Integer> evaluateId, @Param("showStatus") Integer showStatus);

    /**
     * updateTopHotStatus
     * @Title: updateTopHotStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param evaluateId
     * @param topHotStatus
     * @return void
     * @throws
     */
    void updateTopHotStatus(@Param("enterpriseId") Integer enterpriseId, @Param("evaluateId") Integer evaluateId, @Param("topHotStatus") Integer topHotStatus);

    /**
     * updateMaliciousStatus
     * @Title: updateMaliciousStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param evaluateId
     * @param maliciousStatus
     * @return void
     * @throws
     */
    void updateMaliciousStatus(@Param("enterpriseId") Integer enterpriseId, @Param("evaluateId") Integer evaluateId, @Param("maliciousStatus") Integer maliciousStatus);

    /**
     * getTotalChart
     * @Title: getTotalChart
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param startTime
     * @param endTime
     * @param storeIdList
     * @return java.util.Map<java.lang.String,java.lang.Long>
     * @throws
     */
    Map<String, Long> getTotalChart(@Param("enterpriseId") Integer enterpriseId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("ids") List<Integer> storeIdList);

    /**
     * getGoodsCountChart
     * @Title: getGoodsCountChart
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param startTime
     * @param endTime
     * @param storeIdList
     * @return java.util.Map<java.lang.String,java.lang.Long>
     * @throws
     */
    Map<String, Long> getGoodsCountChart(@Param("enterpriseId") Integer enterpriseId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("ids") List<Integer> storeIdList);

    /**
     * getStoreCountChart
     * @Title: getStoreCountChart
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param startTime
     * @param endTime
     * @param storeIdList
     * @return java.util.Map<java.lang.String,java.lang.Long>
     * @throws
     */
    Map<String, Long> getStoreCountChart(@Param("enterpriseId") Integer enterpriseId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("ids") List<Integer> storeIdList);

    /**
     * getServiceCountChart
     * @Title: getServiceCountChart
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param startTime
     * @param endTime
     * @param storeIdList
     * @return java.util.Map<java.lang.String,java.lang.Long>
     * @throws
     */
    Map<String, Long> getServiceCountChart(@Param("enterpriseId") Integer enterpriseId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("ids") List<Integer> storeIdList);

    /**
     * listTrendEvaluate
     * @Title: listTrendEvaluate
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param startTime
     * @param endTime
     * @param goodEvaluateStatus
     * @param storeIdList
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     * @throws
     */
    List<Map<String, Object>> listTrendEvaluate(@Param("enterpriseId") Integer enterpriseId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("goodEvaluateStatus") Integer goodEvaluateStatus, @Param("ids") List<Integer> storeIdList);

    /**
     * listStoreScore
     * @Title: listStoreScore
     * @Description:
     * @author zhiwj
     * @param storeScoreQO
     * @return com.github.pagehelper.Page<java.util.Map<java.lang.String,java.lang.Object>>
     * @throws
     */
    Page<Map<String, Object>> listStoreScore(StoreScoreQO storeScoreQO);

    /**
     * getTotalStoreScore
     * @Title: getTotalStoreScore
     * @Description:
     * @author zhiwj
     * @param storeScoreQO
     * @return java.util.Map<java.lang.String,java.lang.Object>
     * @throws
     */
    Map<String, Object> getTotalStoreScore(StoreScoreQO storeScoreQO);

    /**
     * listTrend
     * @Title: listTrend
     * @Description:
     * @author zhiwj
     * @param storeTrendQO
     * @return com.github.pagehelper.Page<java.util.Map<java.lang.String,java.lang.Object>>
     * @throws
     */
    Page<Map<String, Object>> listTrend(StoreTrendQO storeTrendQO);

    /**
     * getEvaluateByOrderId
     * @Title: getEvaluateByOrderId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param orderId
     * @return com.gic.evaluate.entity.TabEvaluate
     * @throws
     */
    TabEvaluate getEvaluateByOrderId(@Param("enterpriseId") Integer enterpriseId, @Param("orderId") Long orderId);

    /**
     * listEvaluateTrendByYear
     * @Title: listEvaluateTrendByYear
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param replyType
     * @return java.util.Map<java.lang.String,java.util.Map<java.lang.String,java.lang.Object>>
     * @throws
     */
    @MapKey("statisticsDate")
    Map<String, Map<String, Object>> listEvaluateTrendByYear(@Param("enterpriseId") Integer enterpriseId, @Param("replyType") Integer replyType);

    /**
     * listEvaluateTrendByMonth
     * @Title: listEvaluateTrendByMonth
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param replyType
     * @param firstDate
     * @return java.util.Map<java.lang.String,java.util.Map<java.lang.String,java.lang.Object>>
     * @throws
     */
    @MapKey("statisticsDate")
    Map<String, Map<String, Object>> listEvaluateTrendByMonth(@Param("enterpriseId") Integer enterpriseId, @Param("replyType") Integer replyType, @Param("firstDate") Date firstDate);

    /**
     * listEvaluateTrendByDay
     * @Title: listEvaluateTrendByDay
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param replyType
     * @param firstDate
     * @return java.util.Map<java.lang.String,java.util.Map<java.lang.String,java.lang.Object>>
     * @throws
     */
    @MapKey("statisticsDate")
    Map<String, Map<String, Object>> listEvaluateTrendByDay(@Param("enterpriseId") Integer enterpriseId, @Param("replyType") Integer replyType, @Param("firstDate") Date firstDate);

    /**
     * getStoreTopHot
     * @Title: getStoreTopHot
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeId
     * @return java.lang.Integer
     * @throws
     */
    Integer getStoreTopHot(@Param("enterpriseId") Integer enterpriseId, @Param("storeId") Integer storeId);

    /**
     * listAllEvaluateId
     * @Title: listAllEvaluateId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return java.util.List<java.lang.Integer>
     * @throws
     */
    List<Integer> listAllEvaluateId(@Param("enterpriseId") Integer enterpriseId);

    /**
     * getStoreHasTopHot
     * @Title: getStoreHasTopHot
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeIdList
     * @return java.util.Set<java.lang.Integer>
     * @throws
     */
    Set<Integer> getStoreHasTopHot(@Param("enterpriseId") Integer enterpriseId, @Param("ids") List<Integer> storeIdList);

    /**
     * countEvaluateTrend
     * @Title: countEvaluateTrend
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param startTime
     * @param endTime
     * @param storeIdList
     * @param quickReplyType
     * @return java.lang.Integer
     * @throws
     */
    Integer countEvaluateTrend(@Param("enterpriseId") Integer enterpriseId, @Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("ids") List<Integer> storeIdList, @Param("quickReplyType") Integer quickReplyType);

    /**
     * queryEvaluateCountByStoreId
     * @Title: queryEvaluateCountByStoreId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeIdList
     * @return java.util.Map<java.lang.Integer,java.util.Map<java.lang.String,java.lang.Object>>
     * @throws
     */
    @MapKey("storeId")
    Map<Integer,Map<String,Object>> queryEvaluateCountByStoreId(@Param("enterpriseId") Integer enterpriseId, @Param("ids") List<Integer> storeIdList);

    /**
     * getEvaluateByOrderNumber
     * @Title: getEvaluateByOrderNumber
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param orderNumber
     * @return com.gic.evaluate.entity.TabEvaluate
     * @throws
     */
    TabEvaluate getEvaluateByOrderNumber(@Param("enterpriseId") Integer enterpriseId, @Param("orderNumber") String orderNumber);
}