package com.gic.evaluate.service.impl;

import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constants.Constants;
import com.gic.evaluate.constant.ComputeEnum;
import com.gic.evaluate.dao.mapper.TabEvaluateMapper;
import com.gic.evaluate.dto.EvaluateDTO;
import com.gic.evaluate.entity.TabEvaluate;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.qo.StoreScoreQO;
import com.gic.evaluate.qo.StoreTrendQO;
import com.gic.evaluate.service.EvaluateService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 9:54
 */
@Service
public class EvaluateServiceImpl implements EvaluateService {

    /**
     * 产品要求查4年
     */
    static Integer YEAR_NUM = 4;

    /**
     * 月数
     */
    static Integer MONTH_NUM = 4;

    @Autowired
    private TabEvaluateMapper tabEvaluateMapper;
    private Logger logger = LogManager.getLogger(EvaluateServiceImpl.class);

    @Override
    public Page<TabEvaluate> listEvaluate(EvaluateQO evaluateQO) {
        PageHelper.startPage(evaluateQO.getCurrentPage(), evaluateQO.getPageSize());
        if (evaluateQO.getGoodsType() != null) {
            evaluateQO.setGoodsCompute(ComputeEnum.parseCodeToCompute(evaluateQO.getGoodsType()));
        }
        if (evaluateQO.getStoreType() != null) {
            evaluateQO.setStoreCompute(ComputeEnum.parseCodeToCompute(evaluateQO.getStoreType()));
        }
        if (evaluateQO.getServiceType() != null) {
            evaluateQO.setServiceCompute(ComputeEnum.parseCodeToCompute(evaluateQO.getServiceType()));
        }
        return tabEvaluateMapper.listEvaluate(evaluateQO);
    }

    @Override
    public Page<TabEvaluate> listEvaluateNearByStore(EvaluateQO evaluateQO) {
        PageHelper.startPage(evaluateQO.getCurrentPage(), evaluateQO.getPageSize());
        evaluateQO.setEndTime(DateUtil.dateToStr(DateUtil.getStartTimeOfDay(), DateUtil.FORMAT_DATETIME_19));
        return tabEvaluateMapper.listEvaluate(evaluateQO);
    }

    @Override
    public Integer countEvaluate(EvaluateQO evaluateQO) {
        if (evaluateQO.getGoodsType() != null) {
            evaluateQO.setGoodsCompute(ComputeEnum.parseCodeToCompute(evaluateQO.getGoodsType()));
        }
        if (evaluateQO.getStoreType() != null) {
            evaluateQO.setStoreCompute(ComputeEnum.parseCodeToCompute(evaluateQO.getStoreType()));
        }
        if (evaluateQO.getServiceType() != null) {
            evaluateQO.setServiceCompute(ComputeEnum.parseCodeToCompute(evaluateQO.getServiceType()));
        }
        return tabEvaluateMapper.countEvaluate(evaluateQO);
    }

    @Override
    public TabEvaluate getEvaluate(Integer enterpriseId, Integer evaluateId) {
        TabEvaluate evaluate = this.tabEvaluateMapper.selectByPrimaryKey(evaluateId);
        if (evaluate != null && Constants.NORMAL_STATUS.equals(evaluate.getStatus()) && evaluate.getEnterpriseId().equals(enterpriseId)) {
            return evaluate;
        }
        return null;
    }

    @Override
    public Integer saveEvaluate(EvaluateDTO evaluateDTO) {
        TabEvaluate evaluate = EntityUtil.changeEntityByJSON(TabEvaluate.class, evaluateDTO);
        evaluate.setStatus(Constants.NORMAL_STATUS);
        Date date = new Date();
        evaluate.setEvaluateTime(date);
        evaluate.setCreateTime(date);
        tabEvaluateMapper.insertSelective(evaluate);
        return evaluate.getEvaluateId();
    }

    @Override
    public void updateShowStatus(Integer enterpriseId, List<Integer> evaluateId, Integer showStatus) {
        tabEvaluateMapper.updateShowStatus(enterpriseId, evaluateId, showStatus);
    }

    @Override
    public void updateTopHotStatus(Integer enterpriseId, Integer evaluateId, Integer topHotStatus) {
        tabEvaluateMapper.updateTopHotStatus(enterpriseId, evaluateId, topHotStatus);
    }

    @Override
    public void updateMaliciousStatus(Integer enterpriseId, Integer evaluateId, Integer maliciousStatus) {
        TabEvaluate tabEvaluate = new TabEvaluate();
        tabEvaluate.setEvaluateId(evaluateId);
        tabEvaluate.setMaliciousStatus(maliciousStatus);
        if (Constants.NORMAL_STATUS.equals(maliciousStatus)) {
            tabEvaluate.setMaliciousStatusTime(new Date());
        }
        tabEvaluateMapper.updateByPrimaryKeySelective(tabEvaluate);
        //        tabEvaluateMapper.updateMaliciousStatus(enterpriseId, evaluateId, maliciousStatus);
    }

    @Override
    public Map<String, Long> getTotalCountChart(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        return tabEvaluateMapper.getTotalChart(enterpriseId, startTime, endTime, storeIdList);
    }

    @Override
    public Map<String, Long> getGoodsCountChart(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        return tabEvaluateMapper.getGoodsCountChart(enterpriseId, startTime, endTime, storeIdList);
    }

    @Override
    public Map<String, Long> getStoreCountChart(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        return tabEvaluateMapper.getStoreCountChart(enterpriseId, startTime, endTime, storeIdList);
    }

    @Override
    public Map<String, Long> getServiceCountChart(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        return tabEvaluateMapper.getServiceCountChart(enterpriseId, startTime, endTime, storeIdList);
    }

    @Override
    public List<Map<String, Object>> listTrendTotalEvaluate(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        return tabEvaluateMapper.listTrendEvaluate(enterpriseId, startTime, endTime, null, storeIdList);
    }

    @Override
    public List<Map<String, Object>> listTrendGoodEvaluate(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        return tabEvaluateMapper.listTrendEvaluate(enterpriseId, startTime, endTime, Constants.NORMAL_STATUS, storeIdList);
    }

    @Override
    public Page<Map<String, Object>> listStoreScore(StoreScoreQO storeScoreQO) {
        PageHelper.startPage(storeScoreQO);
        return tabEvaluateMapper.listStoreScore(storeScoreQO);
    }

    @Override
    public Map<String, Object> getTotalStoreScore(StoreScoreQO storeScoreQO) {
        PageHelper.startPage(storeScoreQO);
        return tabEvaluateMapper.getTotalStoreScore(storeScoreQO);
    }

    @Override
    public Page<Map<String, Object>> listTrend(StoreTrendQO storeTrendQO) {
        PageHelper.startPage(storeTrendQO);
        return tabEvaluateMapper.listTrend(storeTrendQO);
    }

    @Override
    public TabEvaluate getEvaluateByOrderId(Integer enterpriseId, Long orderId) {
        return tabEvaluateMapper.getEvaluateByOrderId(enterpriseId, orderId);
    }

    @Override
    public void updateEvaluate(EvaluateDTO evaluateDTO) {
        TabEvaluate evaluate = EntityUtil.changeEntityByJSON(TabEvaluate.class, evaluateDTO);
        tabEvaluateMapper.updateByPrimaryKeySelective(evaluate);
    }

//    @Override
//    public List<Map<String, Object>> evaluateGoodQuickChart(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
//        return tabEvaluateMapper.evaluateQuickChart(enterpriseId, startTime, endTime, QuickReplyTypeEnum.GOOD.getCode(), storeIdList);
//    }
//
//    @Override
//    public List<Map<String, Object>> evaluateBadQuickChart(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
//        return tabEvaluateMapper.evaluateQuickChart(enterpriseId, startTime, endTime, QuickReplyTypeEnum.BAD.getCode(), storeIdList);
//    }

    @Override
    public List<Map<String, Object>> listEvaluateTrendByYear(Integer enterpriseId, Integer replyType) {
        Map<String, Map<String, Object>> byYearMap = tabEvaluateMapper.listEvaluateTrendByYear(enterpriseId, replyType);
        List<Map<String, Object>> list = new ArrayList<>();
        List<String> yearList = getYear();
        for (String year : yearList) {
            Map<String, Object> stringObjectMap = byYearMap.get(year);
            if (stringObjectMap != null) {
                list.add(stringObjectMap);
            } else {
                Map<String, Object> stringObjectHashMap = new HashMap<>();
                stringObjectHashMap.put("statisticsDate", year);
                stringObjectHashMap.put("count", 0);
                list.add(stringObjectHashMap);
            }
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> listEvaluateTrendByMonth(Integer enterpriseId, Integer replyType) {
        int year = DateUtil.getYear(new Date());
        String s = year + "-01-01 00:00:00";
        Map<String, Map<String, Object>> byMonthMap;
        List<Map<String, Object>> list = new ArrayList<>();
        try {
            byMonthMap = tabEvaluateMapper.listEvaluateTrendByMonth(enterpriseId, replyType, new SimpleDateFormat(DateUtil.FORMAT_DATETIME_19).parse(s));
            List<String> monthList = getMonth();
            for (String month : monthList) {
                Map<String, Object> stringObjectMap = byMonthMap.get(month);
                if (stringObjectMap != null) {
                    list.add(stringObjectMap);
                } else {
                    Map<String, Object> stringObjectHashMap = new HashMap<>();
                    stringObjectHashMap.put("statisticsDate", month);
                    stringObjectHashMap.put("count", 0);
                    list.add(stringObjectHashMap);
                }
            }
        } catch (ParseException e) {
            logger.info("解析错误", e);
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> listEvaluateTrendByDay(Integer enterpriseId, Integer replyType) {
        Map<String, Map<String, Object>> byDayMap;
        List<Map<String, Object>> list = new ArrayList<>();
        try {
            byDayMap = tabEvaluateMapper.listEvaluateTrendByDay(enterpriseId, replyType, DateUtil.addDay(new Date(), -30));
            List<String> dayList = getDayList();
            for (String day : dayList) {
                Map<String, Object> stringObjectMap = byDayMap.get(day);
                if (stringObjectMap != null) {
                    list.add(stringObjectMap);
                } else {
                    Map<String, Object> stringObjectHashMap = new HashMap<>();
                    stringObjectHashMap.put("statisticsDate", day);
                    stringObjectHashMap.put("count", 0);
                    list.add(stringObjectHashMap);
                }
            }
        } catch (ParseException e) {
            logger.info("解析错误", e);
        }
        return list;
    }

    @Override
    public Integer getStoreTopHot(Integer enterpriseId, Integer storeId) {
        return tabEvaluateMapper.getStoreTopHot(enterpriseId, storeId);
    }

    @Override
    public List<Integer> listAllEvaluateId(Integer enterpriseId) {
        return tabEvaluateMapper.listAllEvaluateId(enterpriseId);
    }

    @Override
    public Set<Integer> getStoreIdHasTopHot(Integer enterpriseId, List<Integer> storeIdList) {
        return tabEvaluateMapper.getStoreHasTopHot(enterpriseId, storeIdList);
    }

    @Override
    public Integer countEvaluateTrend(Integer enterpriseId, Date startTime, Date endTime, List<Integer> storeIdList, Integer quickReplyType) {
        return tabEvaluateMapper.countEvaluateTrend(enterpriseId, startTime, endTime, storeIdList, quickReplyType);
    }

    @Override
    public boolean hasMaliciousStatusBeforeHours(Integer evaluateId, Integer badEvaluateTagType) {
        TabEvaluate tabEvaluate = this.tabEvaluateMapper.selectByPrimaryKey(evaluateId);
        if (tabEvaluate != null && Constants.NORMAL_STATUS.equals(tabEvaluate.getMaliciousStatus())) {
            if (badEvaluateTagType == 1) {
                // 永久禁止
                return true;
            } else {
                // 72小时禁止
                Date maliciousStatusTime = tabEvaluate.getMaliciousStatusTime();
                if (maliciousStatusTime != null) {
                    return DateUtil.compareDate(DateUtil.addNumForDay(maliciousStatusTime, 3), new Date());
                } else {
                    return false;
                }
            }
        } else {
            return false;
        }
    }

    @Override
    public Map<Integer, Map<String, Object>> queryEvaluateCountByStoreId(Integer enterpriseId, List<Integer> storeIdList) {
        return this.tabEvaluateMapper.queryEvaluateCountByStoreId(enterpriseId, storeIdList);
    }

    @Override
    public TabEvaluate getEvaluateByOrderNumber(Integer enterpriseId, String orderNumber) {
        return this.tabEvaluateMapper.getEvaluateByOrderNumber(enterpriseId, orderNumber);
    }

    private List<String> getYear() {
        List<String> list = new ArrayList<>();
        int year = DateUtil.getYear(new Date());
        for (int i = 0; i <= YEAR_NUM; i++) {
            list.add((year - i) + "");
        }
        return list;
    }

    private List<String> getMonth() {
        List<String> list = new ArrayList<>();
        int year = DateUtil.getYear(new Date());
        DecimalFormat decimalFormat = new DecimalFormat("00");
        for (int i = 1; i <= MONTH_NUM; i++) {
            String month = year + "-" + decimalFormat.format(i);
            list.add(month);
        }
        return list;
    }

    private List<String> getDayList() throws ParseException {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date dateOne = DateUtil.addDay(new Date(), -30);
        Date dateTwo = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateOne);

        List<String> dateList = new ArrayList<>();
        while (calendar.getTime().before(dateTwo)) {
            dateList.add(dateFormat.format(calendar.getTime()));
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }
        dateList.add(dateFormat.format(dateTwo));
        return dateList;
    }
}
