package com.gic.evaluate.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.constant.ReplyerTypeEnum;
import com.gic.evaluate.dao.mapper.TabProblemReplyMapper;
import com.gic.evaluate.dto.ProblemMemberReplyDTO;
import com.gic.evaluate.dto.ProblemReplyDTO;
import com.gic.evaluate.dto.ProblemUserReplyDTO;
import com.gic.evaluate.entity.TabProblemImg;
import com.gic.evaluate.entity.TabProblemReply;
import com.gic.evaluate.service.ProblemImgService;
import com.gic.evaluate.service.ProblemReplyService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-02 11:37
 */
@Service
public class ProblemReplyServiceImpl implements ProblemReplyService {

    @Autowired
    private TabProblemReplyMapper tabProblemReplyMapper;
    @Autowired
    private ProblemImgService problemImgService;

    @Override
    public List<ProblemReplyDTO> listReply(Integer problemId) {
        List<TabProblemReply> replyList = tabProblemReplyMapper.listReply(problemId);
        if (CollectionUtils.isEmpty(replyList)) {
            return Collections.emptyList();
        }
        List<ProblemReplyDTO> problemReplyDTOList = EntityUtil.changeEntityListByJSON(ProblemReplyDTO.class, replyList);
        List<TabProblemImg> imgList = problemImgService.listImageByProblemId(problemId);
        Map<Integer, List<String>> map = Optional.of(imgList).orElse(Collections.emptyList())
                .stream().filter(e -> e.getProblemReplyId() != null)
                .collect(Collectors.groupingBy(TabProblemImg::getProblemReplyId, Collectors.mapping(TabProblemImg::getImgUrl, Collectors.toList())));
        for (ProblemReplyDTO problemReplyDTO : problemReplyDTOList) {
            problemReplyDTO.setImgList(map.get(problemReplyDTO.getProblemReplyId()));
        }
        return problemReplyDTOList;
    }

    @Override
    public void save(ProblemUserReplyDTO problemUserReplyDTO) {
        TabProblemReply tabProblemReply = new TabProblemReply();
        tabProblemReply.setEnterpriseId(problemUserReplyDTO.getEnterpriseId());
        tabProblemReply.setProblemId(problemUserReplyDTO.getProblemId());
        tabProblemReply.setReplyerType(ReplyerTypeEnum.USER.getCode());
        tabProblemReply.setReplyContent(problemUserReplyDTO.getReplyContent());
        tabProblemReply.setUserId(problemUserReplyDTO.getUserId());
        tabProblemReply.setUserName(problemUserReplyDTO.getUserName());
        tabProblemReply.setStatus(Constants.OPEN);
        tabProblemReply.setCreateTime(new Date());
        tabProblemReplyMapper.insertSelective(tabProblemReply);
    }

    @Override
    public int save(ProblemMemberReplyDTO problemMemberReplyDTO) {
        TabProblemReply problemReply = new TabProblemReply();
        problemReply.setEnterpriseId(problemMemberReplyDTO.getEnterpriseId());
        problemReply.setProblemId(problemMemberReplyDTO.getProblemId());
        problemReply.setReplyerType(ReplyerTypeEnum.MEMBER.getCode());
        problemReply.setReplyContent(problemMemberReplyDTO.getReplyContent());
        problemReply.setEcuId(problemMemberReplyDTO.getEcuId());
        problemReply.setAcuId(problemMemberReplyDTO.getAcuId());
        problemReply.setMemberId(problemMemberReplyDTO.getMemberId());
        problemReply.setMemberName(problemMemberReplyDTO.getMemberName());
        problemReply.setMemberImgUrl(problemMemberReplyDTO.getMemberImgUrl());
        problemReply.setStatus(Constants.OPEN);
        problemReply.setCreateTime(new Date());
        tabProblemReplyMapper.insertSelective(problemReply);
        return problemReply.getProblemReplyId();
    }

    @Override
    public void update(ProblemReplyDTO problemReplyDTO) {
        TabProblemReply tabProblemReply = EntityUtil.changeEntityByJSON(TabProblemReply.class, problemReplyDTO);
        tabProblemReplyMapper.updateByPrimaryKeySelective(tabProblemReply);
    }

    @Override
    public Map<Integer, String> firstReplyByProblemId(List<Integer> problemIdList) {
        List<TabProblemReply> problemReplyList = tabProblemReplyMapper.firstReplyByProblemId(problemIdList);
        return Optional.ofNullable(problemReplyList).orElse(Collections.emptyList()).stream().collect(Collectors.toMap(TabProblemReply::getProblemId, TabProblemReply::getReplyContent));
    }
}
