package com.gic.evaluate.service.impl;

import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.constant.ReplyEnum;
import com.gic.evaluate.dao.mapper.TabProblemMapper;
import com.gic.evaluate.dto.ProblemDTO;
import com.gic.evaluate.entity.TabProblem;
import com.gic.evaluate.qo.ProblemQO;
import com.gic.evaluate.service.ProblemService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-02 11:17
 */
@Service
public class ProblemServiceImpl implements ProblemService {

    @Autowired
    private TabProblemMapper tabProblemMapper;

    @Override
    public Page<TabProblem> listProblem(ProblemQO problemQO) {
        PageHelper.startPage(problemQO.getCurrentPage(), problemQO.getPageSize());
        return tabProblemMapper.listProblem(problemQO);
    }

    @Override
    public TabProblem getProblem(Integer problemId) {
        return tabProblemMapper.selectByPrimaryKey(problemId);
    }

    @Override
    public void updateProblem(ProblemDTO problemDTO) {
        TabProblem problem = EntityUtil.changeEntityByJSON(TabProblem.class, problemDTO);
        tabProblemMapper.updateByPrimaryKeySelective(problem);
    }

    @Override
    public Integer save(ProblemDTO problemDTO) {
        TabProblem problem = new TabProblem();
        problem.setEnterpriseId(problemDTO.getEnterpriseId());
        problem.setStoreId(problemDTO.getStoreId());
        problem.setStoreName(problemDTO.getStoreName());
        problem.setEcuId(problemDTO.getEcuId());
        problem.setAcuId(problemDTO.getAcuId());
        problem.setMemberId(problemDTO.getMemberId());
        problem.setMemberName(problemDTO.getMemberName());
        problem.setMemberPhone(problemDTO.getMemberPhone());
        problem.setMemberImgUrl(problemDTO.getMemberImgUrl());
        problem.setProblemContent(problemDTO.getProblemContent());
        problem.setImageUrl(problemDTO.getImageUrl());
        problem.setHasOtherImage(problemDTO.getHasOtherImage());
        problem.setProblemStatus(ReplyEnum.WAIT.getCode());
        problem.setMaliciousStatus(Constants.CLOSE);
        problem.setShowStatus(Constants.OPEN);
        problem.setStatus(Constants.OPEN);
        problem.setCreateTime(new Date());
        tabProblemMapper.insertSelective(problem);
        return problem.getProblemId();
    }

    @Override
    public List<ProblemDTO> listProblem(Long acuId) {
        return tabProblemMapper.listProblemByAcuId(acuId);
    }

    @Override
    public void endProblem(Integer problemId) {
        TabProblem problem = new TabProblem();
        problem.setProblemId(problemId);
        problem.setStatus(2);
        tabProblemMapper.updateByPrimaryKeySelective(problem);
    }

    @Override
    public Integer countProblem(ProblemQO problemQO) {
        return tabProblemMapper.countProblem(problemQO);
    }

    @Override
    public boolean hasMaliciousStatusByAcuIdBeforeHours(Long acuId, Integer badSuggestionTagType) {
        Date date = DateUtil.addNumForHour(new Date(), -badSuggestionTagType);
        TabProblem problem = tabProblemMapper.getMaliciousStatusByAcuIdBeforeHours(acuId, date);
        return problem != null;
    }
}
