package com.gic.evaluate.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.constant.ReplyEnum;
import com.gic.evaluate.constant.ReplyerTypeEnum;
import com.gic.evaluate.dto.*;
import com.gic.evaluate.entity.*;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.qo.StoreScoreQO;
import com.gic.evaluate.service.*;
import com.gic.store.service.StoreApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 9:50
 */
@Service("evaluateManageApiService")
public class EvaluateManageApiServiceImpl implements EvaluateManageApiService {

    private Logger logger = LogManager.getLogger(EvaluateManageApiServiceImpl.class);
    @Autowired
    private EvaluateService evaluateService;
    @Autowired
    private EvaluateReplyService evaluateReplyService;
    @Autowired
    private EvaluateQuickService evaluateQuickService;
    @Autowired
    private EvaluateImgService evaluateImgService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private EvaluateOrderConfigService evaluateOrderConfigService;
    @Autowired
    private EvaluateTagsService evaluateTagsService;


    @Override
    public ServiceResponse<Page<EvaluateDTO>> listEvaluateForManage(EvaluateQO evaluateQO) {
        com.github.pagehelper.Page<TabEvaluate> evaluateList = evaluateService.listEvaluate(evaluateQO);
        Page<EvaluateDTO> page = PageHelperUtils.changePageHelperToCurrentPage(evaluateList, EvaluateDTO.class);
        List<EvaluateDTO> result = page.getResult();
        if (CollectionUtils.isNotEmpty(result)) {
            List<Integer> storeIdList = result.stream().map(EvaluateDTO::getStoreId).collect(Collectors.toList());
            Set<Integer> storeIdHasTopHot = this.getStoreIdHasTopHot(evaluateQO.getEnterpriseId(), storeIdList);
            for (EvaluateDTO evaluateDTO : result) {
                evaluateDTO.setHasTopHot(storeIdHasTopHot.contains(evaluateDTO.getStoreId()));
            }
        }
        return EnterpriseServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<EvaluateDTO> getEvaluate(Integer enterpriseId, Integer evaluateId) {
        TabEvaluate evaluate = evaluateService.getEvaluate(enterpriseId, evaluateId);
        if (evaluate == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        EvaluateDTO evaluateDTO = EntityUtil.changeEntityByJSON(EvaluateDTO.class, evaluate);

        List<TabEvaluateImg> evaluateImgList = evaluateImgService.listImageByEvaluateId(evaluateDTO.getEvaluateId());
        List<String> imgList = Optional.ofNullable(evaluateImgList).orElse(Collections.emptyList()).stream().map(TabEvaluateImg::getImgUrl).collect(Collectors.toList());
        evaluateDTO.setImgList(imgList);

        List<TabEvaluateReply> tabReplyList = evaluateReplyService.listEvaluateReply(enterpriseId, evaluateId, null);
        List<EvaluateReplyDTO> replyDTOList = EntityUtil.changeEntityListByJSON(EvaluateReplyDTO.class, tabReplyList);
        for (EvaluateReplyDTO evaluateReplyDTO : replyDTOList) {
            List<TabEvaluateImg> tabEvaluateImgs = evaluateImgService.listImageByEvaluateReplyId(evaluateReplyDTO.getEvaluateReplyId());
            List<String> replyImgList = Optional.ofNullable(tabEvaluateImgs).orElse(Collections.emptyList()).stream().map(TabEvaluateImg::getImgUrl).collect(Collectors.toList());
            evaluateReplyDTO.setImgList(replyImgList);
            List<TabEvaluateTags> tagsList = evaluateTagsService.listEvaluateTags(evaluateReplyDTO.getEnterpriseId(), evaluateReplyDTO.getEvaluateId(), evaluateReplyDTO.getEvaluateReplyId());
            evaluateReplyDTO.setQuickReplyTagList(EntityUtil.changeEntityListByJSON(EvaluateTagsDTO.class, tagsList));
        }
        evaluateDTO.setReplyList(replyDTOList);

        EvaluateReplyDTO evaluateReplyDTO = new EvaluateReplyDTO();
        evaluateReplyDTO.setEvaluateId(evaluateDTO.getEvaluateId());
        evaluateReplyDTO.setReplyDate(evaluateDTO.getEvaluateTime());
        evaluateReplyDTO.setReplyerType(ReplyerTypeEnum.MEMBER.getCode());
        evaluateReplyDTO.setReplyContent(evaluateDTO.getEvaluateContent());
        evaluateReplyDTO.setMemberId(evaluateDTO.getMemberId());
        evaluateReplyDTO.setMemberName(evaluateDTO.getMemberName());
        evaluateReplyDTO.setMemberImgUrl(evaluateDTO.getMemberImgUrl());
        evaluateReplyDTO.setImgList(evaluateDTO.getImgList());
        evaluateReplyDTO.setShowStatus(evaluateDTO.getShowStatus());
        List<TabEvaluateTags> tagsList = evaluateTagsService.listEvaluateTags(evaluateDTO.getEnterpriseId(), evaluateDTO.getEvaluateId());
        evaluateReplyDTO.setQuickReplyTagList(EntityUtil.changeEntityListByJSON(EvaluateTagsDTO.class, tagsList));
        List<EvaluateReplyDTO> replyList = evaluateDTO.getReplyList();
        if (replyList == null) {
            replyList = new ArrayList<>();
            evaluateDTO.setReplyList(replyList);
        }
        replyList.add(0, evaluateReplyDTO);


        return EnterpriseServiceResponse.success(evaluateDTO);
    }

    @Override
    public ServiceResponse<Void> updateShowStatus(Integer enterpriseId, List<Integer> evaluateId, Integer showStatus) {
        if (CollectionUtils.isNotEmpty(evaluateId)) {
            if (evaluateId.size() == 1 && evaluateId.get(0) == -1) {
                // 如果传了-1  就是修改全部
                evaluateId = Collections.emptyList();
            }
            this.evaluateService.updateShowStatus(enterpriseId, evaluateId, showStatus);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateReplyShowStatus(Integer enterpriseId, Integer evaluateReplyId, Integer showStatus) {
        this.evaluateReplyService.updateReplyShowStatus(enterpriseId, evaluateReplyId, showStatus);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateTopHotStatus(Integer enterpriseId, Integer evaluateId, Integer topHotStatus) {
        if (Constants.OPEN.equals(topHotStatus)) {
            TabEvaluate evaluate = this.evaluateService.getEvaluate(enterpriseId, evaluateId);
            Integer oldEvaluateId = this.evaluateService.getStoreTopHot(enterpriseId, evaluate.getStoreId());
            this.evaluateService.updateTopHotStatus(enterpriseId, oldEvaluateId, Constants.CLOSE);
        }
        this.evaluateService.updateTopHotStatus(enterpriseId, evaluateId, topHotStatus);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateMaliciousStatus(Integer enterpriseId, Integer evaluateId, Integer maliciousStatus) {
        this.evaluateService.updateMaliciousStatus(enterpriseId, evaluateId, maliciousStatus);
        if (Constants.OPEN.equals(maliciousStatus)) {
            this.evaluateService.updateShowStatus(enterpriseId, Collections.singletonList(evaluateId), Constants.CLOSE);
        } else if (Constants.CLOSE.equals(maliciousStatus)) {
            this.evaluateService.updateShowStatus(enterpriseId, Collections.singletonList(evaluateId), Constants.OPEN);
        }
        TabEvaluate evaluate = this.evaluateService.getEvaluate(enterpriseId, evaluateId);
        if (evaluate != null && evaluate.getStoreId() != null) {
            this.computeStoreScoreEvaluate(enterpriseId, evaluate.getStoreId());
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> replyEvaluate(UserReplyDTO userReplyDTO) {
        if (userReplyDTO.getEvaluateIds() != null) {
            List<Integer> evaluateIdList;
            if (userReplyDTO.getEvaluateIds().length == 1 && userReplyDTO.getEvaluateIds()[0] == -1) {
                // 如果传了-1  就是修改全部
                evaluateIdList = evaluateService.listAllEvaluateId(userReplyDTO.getEnterpriseId());
            } else {
                evaluateIdList = Arrays.asList(userReplyDTO.getEvaluateIds());
            }
            for (Integer evaluateId : evaluateIdList) {
                TabEvaluate evaluate = evaluateService.getEvaluate(userReplyDTO.getEnterpriseId(), evaluateId);
                if (!ReplyEnum.REPLIED.getCode().equals(evaluate.getReplyStatus())) {
                    userReplyDTO.setEvaluateId(evaluateId);
                    this.evaluateReplyService.saveUserReplyEvaluate(userReplyDTO);
                    EvaluateDTO evaluateDTO = new EvaluateDTO();
                    evaluateDTO.setEvaluateId(evaluateId);
                    evaluateDTO.setReplyStatus(ReplyEnum.REPLIED.getCode());
                    this.evaluateService.updateEvaluate(evaluateDTO);
                } else {
                    logger.info("不能回复已回复的评价, evaluateId:{}", evaluateId);
                }
            }
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> editReplyEvaluate(UserReplyDTO userReplyDTO) {
        evaluateReplyService.update(userReplyDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> delReply(Integer evaluateReplyId) {
        this.evaluateReplyService.delReply(evaluateReplyId);
        return EnterpriseServiceResponse.success();
    }

    private Set<Integer> getStoreIdHasTopHot(Integer enterpriseId, List<Integer> storeIdList) {
        return this.evaluateService.getStoreIdHasTopHot(enterpriseId, storeIdList);
    }

    @Override
    public ServiceResponse<List<EvaluateQuickDTO>> listEvaluateQuickReply(Integer enterpriseId, Integer quickReplyType) {
        List<TabEvaluateQuick> evaluateQuickList = this.evaluateQuickService.listEvaluateQuick(enterpriseId, quickReplyType);
        return EnterpriseServiceResponse.success(EntityUtil.changeEntityListByJSON(EvaluateQuickDTO.class, evaluateQuickList));
    }

    @Override
    public ServiceResponse<Integer> countEvaluate(EvaluateQO evaluateQO) {
        Integer count = evaluateService.countEvaluate(evaluateQO);
        return EnterpriseServiceResponse.success(count);
    }

    @Override
    public ServiceResponse<Integer> computeStoreScoreEvaluate(String msg) {
        List<EnterpriseDTO> enterpriseList = enterpriseApiService.listEnterpriseByName(null).getResult();
        List<Integer> enterpriseIds = Optional.ofNullable(enterpriseList).orElse(Collections.emptyList()).stream().map(EnterpriseDTO::getEnterpriseId).collect(Collectors.toList());
        EvaluateQO evaluateQO = new EvaluateQO();
        evaluateQO.setMaliciousStatus(Constants.CLOSE);
        evaluateQO.setShowStatus(Constants.OPEN);
        for (Integer enterpriseId : enterpriseIds) {
            TabEvaluateOrderConfig evaluateOrderConfig = this.evaluateOrderConfigService.getEvaluateOrderConfig(enterpriseId);
            StoreScoreQO storeScoreQO = new StoreScoreQO();
            storeScoreQO.setEnterpriseId(enterpriseId);
            storeScoreQO.setCurrentPage(1);
            storeScoreQO.setEndTime(DateUtil.dateToStr(DateUtil.getStartTimeOfDay(), DateUtil.FORMAT_DATETIME_19));
            storeScoreQO.setPageSize(Integer.MAX_VALUE);
            List<Map<String, Object>> list = this.evaluateService.listStoreScore(storeScoreQO);
            for (Map<String, Object> stringObjectMap : list) {
                Object goodsScore = stringObjectMap.get("goodsScore");
                Object storeScore = stringObjectMap.get("storeScore");
                Object serviceScore = stringObjectMap.get("serviceScore");

                int count = 0;
                double sum = 0;
                if (Constants.OPEN.equals(evaluateOrderConfig.getGoods()) && goodsScore != null) {
                    count++;
                    sum += Double.valueOf(goodsScore.toString());
                }
                if (Constants.OPEN.equals(evaluateOrderConfig.getStore()) && storeScore != null) {
                    count++;
                    sum += Double.valueOf(storeScore.toString());
                }
                if (Constants.OPEN.equals(evaluateOrderConfig.getService()) && serviceScore != null) {
                    count++;
                    sum += Double.valueOf(serviceScore.toString());
                }
                Object storeId = stringObjectMap.get("storeId");
                if (storeId != null) {
                    Integer storeIdInt = Integer.valueOf(storeId.toString());
                    if (count > 0) {
                        evaluateQO.setStoreIdList(Collections.singletonList(storeIdInt));
                        Integer evaluateCount = this.evaluateService.countEvaluate(evaluateQO);
                        storeApiService.updateScore(enterpriseId, storeIdInt, sum / count, evaluateCount);
                    } else {
                        // 如果是没有评价过的门店 默认给10分
                        storeApiService.updateScore(enterpriseId, storeIdInt, 10, 0);
                    }
                }
            }
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> computeStoreScoreEvaluate(Integer enterpriseId, Integer storeId) {
        TabEvaluateOrderConfig evaluateOrderConfig = this.evaluateOrderConfigService.getEvaluateOrderConfig(enterpriseId);
        EvaluateQO evaluateQO = new EvaluateQO();
        evaluateQO.setMaliciousStatus(Constants.CLOSE);
        evaluateQO.setShowStatus(Constants.OPEN);

        StoreScoreQO storeScoreQO = new StoreScoreQO();
        storeScoreQO.setEnterpriseId(enterpriseId);
        storeScoreQO.setStoreId(storeId);
        storeScoreQO.setCurrentPage(1);
        storeScoreQO.setEndTime(DateUtil.dateToStr(DateUtil.getStartTimeOfDay(), DateUtil.FORMAT_DATETIME_19));
        storeScoreQO.setPageSize(Integer.MAX_VALUE);
        List<Map<String, Object>> list = this.evaluateService.listStoreScore(storeScoreQO);
        for (Map<String, Object> stringObjectMap : list) {
            Object goodsScore = stringObjectMap.get("goodsScore");
            Object storeScore = stringObjectMap.get("storeScore");
            Object serviceScore = stringObjectMap.get("serviceScore");

            int count = 0;
            double sum = 0;
            if (Constants.OPEN.equals(evaluateOrderConfig.getGoods()) && goodsScore != null) {
                count++;
                sum += Double.valueOf(goodsScore.toString());
            }
            if (Constants.OPEN.equals(evaluateOrderConfig.getStore()) && storeScore != null) {
                count++;
                sum += Double.valueOf(storeScore.toString());
            }
            if (Constants.OPEN.equals(evaluateOrderConfig.getService()) && serviceScore != null) {
                count++;
                sum += Double.valueOf(serviceScore.toString());
            }
            if (count > 0) {
                if (storeId != null) {
                    evaluateQO.setStoreIdList(Collections.singletonList(storeId));
                    Integer evaluateCount = this.evaluateService.countEvaluate(evaluateQO);
                    storeApiService.updateScore(enterpriseId, storeId, sum / count, evaluateCount);
                }
            }
        }
        return EnterpriseServiceResponse.success();
    }

}
