package com.gic.evaluate.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.constant.OrderReplyEnum;
import com.gic.evaluate.constant.ReplyEnum;
import com.gic.evaluate.dto.*;
import com.gic.evaluate.entity.*;
import com.gic.evaluate.qo.EvaluateOutQO;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.qo.OrderStatusQO;
import com.gic.evaluate.service.*;
import com.gic.member.api.dto.AcuDetailDTO;
import com.gic.member.api.dto.MemberUserDTO;
import com.gic.member.api.service.CuDetailService;
import com.gic.member.api.service.MemberUserApiService;
import com.gic.order.api.dto.OrderListDetailDTO;
import com.gic.order.api.service.OrderApiService;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.StoreApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 15:27
 */
@Service("evaluateOutApiService")
public class EvaluateOutApiServiceImpl implements EvaluateOutApiService {

    private Logger logger = LogManager.getLogger(EvaluateOutApiServiceImpl.class);
    
    @Autowired
    private EvaluateService evaluateService;
    @Autowired
    private EvaluateReplyService evaluateReplyService;
    @Autowired
    private EvaluateOrderConfigService evaluateOrderConfigService;
    @Autowired
    private BadEvaluateConfigService badEvaluateConfigService;
    @Autowired
    private EvaluateImgService evaluateImgService;
    @Autowired
    private CuDetailService cuDetailService;
    @Autowired
    private MemberUserApiService memberUserApiService;
    @Autowired
    private OrderApiService orderApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private EvaluateTagsService evaluateTagsService;
    @Override
    public ServiceResponse<Void> saveEvaluate(EvaluateDTO evaluateDTO) {
        logger.info("保存订单评价:{}", JSON.toJSONString(evaluateDTO));
        TabEvaluate evaluate = evaluateService.getEvaluateByOrderId(evaluateDTO.getEnterpriseId(), evaluateDTO.getOrderId());
        if (evaluate != null) {
            return EnterpriseServiceResponse.failure(ErrorCode.DATA_EXISTS);
        }
        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(evaluateDTO.getEnterpriseId());
        OrderListDetailDTO orderListDetailDTO = orderApiService.getOrderDetail(evaluateDTO.getEnterpriseId(), evaluateDTO.getOrderId(), evaluateDTO.getMemberId()).getResult();

        // 判断是否超过评价时间
        if (overLimitTime(evaluateOrderConfig, orderListDetailDTO.getOrderDTO().getReceiptsTime())) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已超越评价期限，无法评价");
        }

        List<OrderGoodsDTO> dtoList = EntityUtil.changeEntityListByJSON(OrderGoodsDTO.class, orderListDetailDTO.getOrderItemList());
        evaluateDTO.setGoodsInfo(JSON.toJSONString(dtoList));

        StoreDTO storeDTO = storeApiService.getStoreByStoreInfoId(evaluateDTO.getEnterpriseId(), Integer.valueOf(orderListDetailDTO.getOrderDTO().getStoreIdOffline())).getResult();
        evaluateDTO.setStoreId(storeDTO.getStoreId());
        evaluateDTO.setStoreInfoId(storeDTO.getStoreInfoId());
        evaluateDTO.setStoreName(storeDTO.getStoreName());
        evaluateDTO.setStoreCode(storeDTO.getStoreCode());
        evaluateDTO.setStoreGroupId(storeDTO.getStoreGroupId());
        evaluateDTO.setStoreGroupName(storeDTO.getStoreGroupName());
        evaluateDTO.setAreaName(storeDTO.getProvinces());

        evaluateDTO.setOrderNumber(orderListDetailDTO.getOrderDTO().getOrderNo());
        evaluateDTO.setReceiptsDate(orderListDetailDTO.getOrderDTO().getReceiptsTime());
        evaluateDTO.setOrderMount(orderListDetailDTO.getOrderDTO().getPayAmount());
        evaluateDTO.setClerkId(orderListDetailDTO.getOrderDTO().getClerkId());
        evaluateDTO.setClerkName(orderListDetailDTO.getOrderDTO().getClerkName());

        evaluateDTO.setReviewStatus(Constants.CLOSE);
        evaluateDTO.setTopHotStatus(Constants.CLOSE);
        evaluateDTO.setMaliciousStatus(Constants.CLOSE);
        evaluateDTO.setShowStatus(Constants.OPEN);
        evaluateDTO.setVisitStatus(0);
        evaluateDTO.setReplyStatus(ReplyEnum.WAIT.getCode());

        // 如果评价指标没开，需要对数据做处理
        if (Constants.CLOSE.equals(evaluateOrderConfig.getGoods())) {
            evaluateDTO.setGoodsScore(null);
        }
        if (Constants.CLOSE.equals(evaluateOrderConfig.getStore())) {
            evaluateDTO.setStoreScore(null);
        }
        if (Constants.CLOSE.equals(evaluateOrderConfig.getService())) {
            evaluateDTO.setServiceScore(null);
        }
        TabBadEvaluateConfig badEvaluateConfig = badEvaluateConfigService.getBadEvaluateConfig(evaluateDTO.getEnterpriseId());
        Integer badEvaluate = isBadEvaluate(evaluateOrderConfig, badEvaluateConfig, evaluateDTO.getGoodsScore(), evaluateDTO.getStoreScore(), evaluateDTO.getServiceScore());
        Integer goodEvaluate = isGoodEvaluate(evaluateOrderConfig, badEvaluateConfig, evaluateDTO.getGoodsScore(), evaluateDTO.getStoreScore(), evaluateDTO.getServiceScore());
        evaluateDTO.setBadEvaluateStatus(badEvaluate);
        evaluateDTO.setGoodEvaluateStatus(goodEvaluate);

        ServiceResponse<MemberUserDTO> memberUserDtoServiceResponse = memberUserApiService.getByMcuId(evaluateDTO.getMemberId(), evaluateDTO.getEnterpriseId());
        if (memberUserDtoServiceResponse.isSuccess()) {

            MemberUserDTO memberUserDTO = memberUserDtoServiceResponse.getResult();
            evaluateDTO.setMemberName(memberUserDTO.getName());
            evaluateDTO.setMemberPhone(memberUserDTO.getMobile());

            if (evaluateDTO.getAcuId() != null) {
                ServiceResponse<AcuDetailDTO> acuDetailResponse = cuDetailService.getAcuDetail(evaluateDTO.getAcuId(), evaluateDTO.getEnterpriseId(), evaluateDTO.getEcuId());
                if (acuDetailResponse.isSuccess() || acuDetailResponse.getResult() != null) {
                    evaluateDTO.setMemberImgUrl(acuDetailResponse.getResult().getHeadUrl());
                }
            }

        }
        // 计算平均分
        double avgScore = getAvgScore(evaluateOrderConfig, evaluateDTO.getGoodsScore(), evaluateDTO.getStoreScore(), evaluateDTO.getServiceScore());
        evaluateDTO.setAvgScore(avgScore);


        Integer evaluateId = evaluateService.saveEvaluate(evaluateDTO);
        evaluateImgService.saveList(evaluateDTO.getEnterpriseId(), evaluateId, null, evaluateDTO.getImgList());

        // 记录快评
        saveQuick(evaluateDTO.getQuickIds(), evaluateDTO.getEnterpriseId(), evaluateId, evaluateDTO.getMemberId());

        return EnterpriseServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> saveReply(MemberReplyDTO memberReplyDTO) {
        logger.info("追评订单评价:{}", JSON.toJSONString(memberReplyDTO));
        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(memberReplyDTO.getEnterpriseId());

        // 判断是否支持
        if (Constants.CLOSE.equals(evaluateOrderConfig.getReview())) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不支持追评");
        }

        if (Constants.OPEN.equals(evaluateOrderConfig.getBadEvaluateTag())) {
            if (this.evaluateService.hasMaliciousStatusBeforeHours(memberReplyDTO.getEvaluateId(), evaluateOrderConfig.getBadEvaluateTagType())) {
                return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不能评论");
            }
        }


        TabEvaluate evaluate = this.evaluateService.getEvaluate(memberReplyDTO.getEnterpriseId(), memberReplyDTO.getEvaluateId());
        if (overLimitTime(evaluateOrderConfig, evaluate.getReceiptsDate())) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已超越评价期限，无法评价");
        }
        // 是否超过次数
        List<TabEvaluateReply> replyList = this.evaluateReplyService.listEvaluateReply(memberReplyDTO.getEnterpriseId(), memberReplyDTO.getEvaluateId(), Constants.OPEN);
        if (CollectionUtils.isNotEmpty(replyList)) {
            if (evaluateOrderConfig.getReviewTimes() <= this.evaluateReplyService.countReviewReplyByMember(memberReplyDTO.getEnterpriseId(), evaluate.getEvaluateId())) {
                return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经超过最大追评次数，无法继续追评");
            }
        }

        ServiceResponse<MemberUserDTO> memberUserDtoServiceResponse = memberUserApiService.getByMcuId(memberReplyDTO.getMemberId(), memberReplyDTO.getEnterpriseId());
        if (memberUserDtoServiceResponse.isSuccess()) {
            memberReplyDTO.setMemberName(memberUserDtoServiceResponse.getResult().getName());
            ServiceResponse<AcuDetailDTO> acuDetailResponse = cuDetailService.getAcuDetail(memberReplyDTO.getAcuId(), memberReplyDTO.getEnterpriseId(), memberReplyDTO.getEcuId());
            if (acuDetailResponse.isSuccess() || acuDetailResponse.getResult() != null) {
                memberReplyDTO.setMemberImgUrl(acuDetailResponse.getResult().getHeadUrl());
            }
        }

        // 保存追评
        Integer evaluateRelayId = evaluateReplyService.saveMemberReplyEvaluate(memberReplyDTO);

        // 更新追评状态

        EvaluateDTO evaluateDTO = new EvaluateDTO();
        evaluateDTO.setEvaluateId(memberReplyDTO.getEvaluateId());
        evaluateDTO.setReplyStatus(ReplyEnum.HAS_NEW.getCode());
        evaluateDTO.setReviewStatus(Constants.OPEN);
        evaluateService.updateEvaluate(evaluateDTO);

        // 保存追评图片
        //        void saveList(Integer enterpriseId, Integer evaluateId, Integer evaluateReplyId, List<String> imgUrlList);
        evaluateImgService.saveList(memberReplyDTO.getEnterpriseId(), memberReplyDTO.getEvaluateId(), evaluateRelayId, memberReplyDTO.getImgList());

        saveQuick(evaluateDTO.getQuickIds(), evaluateDTO.getEnterpriseId(), evaluateDTO.getEvaluateId(), evaluateRelayId, evaluateDTO.getMemberId());

        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<EvaluateDTO>> listEvaluateByStore(Integer enterpriseId, Integer storeId, Integer currentPage, Integer pageSize) {
        EvaluateQO evaluateQO = new EvaluateQO();
        evaluateQO.setEnterpriseId(enterpriseId);
        evaluateQO.setStoreIdList(Collections.singletonList(storeId));
        evaluateQO.setShowStatus(Constants.OPEN);
        evaluateQO.setMaliciousStatus(Constants.CLOSE);
        evaluateQO.setCurrentPage(currentPage);
        evaluateQO.setPageSize(pageSize);
        evaluateQO.setApplet(1);

        com.github.pagehelper.Page<TabEvaluate> topHotEvaluate = evaluateService.listEvaluateNearByStore(evaluateQO);
        if (CollectionUtils.isEmpty(topHotEvaluate)) {
            topHotEvaluate = new com.github.pagehelper.Page<>();
        }

        Page<EvaluateDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(topHotEvaluate, EvaluateDTO.class);
        List<EvaluateDTO> resultList = resultPage.getResult();
        for (EvaluateDTO evaluateDTO : resultList) {

            List<TabEvaluateImg> evaluateImgList = evaluateImgService.listImageByEvaluateId(evaluateDTO.getEvaluateId());
            List<String> imgList = Optional.ofNullable(evaluateImgList).orElse(Collections.emptyList()).stream().map(TabEvaluateImg::getImgUrl).collect(Collectors.toList());
            evaluateDTO.setImgList(imgList);

            List<TabEvaluateTags> tagsList = evaluateTagsService.listEvaluateTags(evaluateDTO.getEnterpriseId(), evaluateDTO.getEvaluateId());

            evaluateDTO.setQuickReplyTagNameList(Optional.ofNullable(tagsList).orElse(Collections.emptyList()).stream().map(TabEvaluateTags::getTagName).collect(Collectors.toList()));
        }


        return EnterpriseServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Page<EvaluateDTO>> getFirstEvaluateByStore(Integer enterpriseId, Integer storeId) {
        return listEvaluateByStore(enterpriseId, storeId, 1, 1);
    }

    private boolean overEvaluateTime(TabEvaluateOrderConfig evaluateOrderConfig, Date receiptsDate) {
        GregorianCalendar calendar = new GregorianCalendar();
        calendar.setTime(receiptsDate);
        calendar.add(Calendar.DAY_OF_MONTH, evaluateOrderConfig.getEvaluateLimitday());
        return DateUtil.compareDate(new Date(), calendar.getTime());
    }

    private boolean overLimitTime(TabEvaluateOrderConfig evaluateOrderConfig, Date receiptsDate) {
        GregorianCalendar calendar = new GregorianCalendar();
        calendar.setTime(receiptsDate);
        calendar.add(Calendar.DAY_OF_MONTH, evaluateOrderConfig.getEvaluateLimitday());

        return DateUtil.compareDate(new Date(), calendar.getTime());
    }

    @Override
    public ServiceResponse<EvaluateDTO> listEvaluateByOrderId(EvaluateOutQO evaluateOutQO) {
        TabEvaluate evaluate = this.evaluateService.getEvaluateByOrderId(evaluateOutQO.getEnterpriseId(), evaluateOutQO.getOrderId());
        if (evaluate == null) {
            if (StringUtils.isNotBlank(evaluateOutQO.getOrderNumber())) {
                evaluate = this.evaluateService.getEvaluateByOrderNumber(evaluateOutQO.getEnterpriseId(), evaluateOutQO.getOrderNumber());
            } else {
                return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
            }
        }

        EvaluateDTO evaluateDTO = EntityUtil.changeEntityByJSON(EvaluateDTO.class, evaluate);

        List<TabEvaluateImg> evaluateImgList = evaluateImgService.listImageByEvaluateId(evaluateDTO.getEvaluateId());
        List<String> imgList = Optional.ofNullable(evaluateImgList).orElse(Collections.emptyList()).stream().map(TabEvaluateImg::getImgUrl).collect(Collectors.toList());
        evaluateDTO.setImgList(imgList);
        // 追评和回复
        List<TabEvaluateReply> evaluateReplyList = this.evaluateReplyService.listEvaluateReply(evaluateDTO.getEnterpriseId(), evaluateDTO.getEvaluateId(), Constants.OPEN);
        List<EvaluateReplyDTO> replyDTOList = EntityUtil.changeEntityListByJSON(EvaluateReplyDTO.class, evaluateReplyList);


        List<TabEvaluateTags> tagsList = evaluateTagsService.listEvaluateTags(evaluateDTO.getEnterpriseId(), evaluateDTO.getEvaluateId());

        evaluateDTO.setQuickReplyTagNameList(Optional.ofNullable(tagsList).orElse(Collections.emptyList()).stream().map(TabEvaluateTags::getTagName).collect(Collectors.toList()));
        // 追评和回复的图片
        for (EvaluateReplyDTO replyDTO : replyDTOList) {
            List<TabEvaluateImg> evaluateReplyImgList = evaluateImgService.listImageByEvaluateReplyId(replyDTO.getEvaluateReplyId());
            List<String> replyImgList = Optional.ofNullable(evaluateReplyImgList).orElse(Collections.emptyList()).stream().map(TabEvaluateImg::getImgUrl).collect(Collectors.toList());
            replyDTO.setImgList(replyImgList);

            List<TabEvaluateTags> replyTagsList = evaluateTagsService.listEvaluateTags(replyDTO.getEnterpriseId(), replyDTO.getEvaluateId(), replyDTO.getEvaluateReplyId());
            replyDTO.setQuickReplyTagNameList(Optional.ofNullable(replyTagsList).orElse(Collections.emptyList()).stream().map(TabEvaluateTags::getTagName).collect(Collectors.toList()));
        }
        evaluateDTO.setReplyList(replyDTOList);

        return EnterpriseServiceResponse.success(evaluateDTO);
    }

    @Override
    public ServiceResponse<Map<Long, Integer>> queryRelayStatusByOrderId(Integer enterpriseId, List<OrderStatusQO> orderIdList) {
        Map<Long, Integer> resultMap = new HashMap<>();
        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(enterpriseId);


        for (OrderStatusQO statusQO : orderIdList) {
            TabEvaluate evaluate = this.evaluateService.getEvaluateByOrderId(enterpriseId, statusQO.getOrderId());

            OrderListDetailDTO orderListDetailDTO = orderApiService.getOrderDetail(enterpriseId, statusQO.getOrderId(), statusQO.getMemberId()).getResult();
            if (orderListDetailDTO != null) {

                // 判断是否超过评价时间
                if (overLimitTime(evaluateOrderConfig, orderListDetailDTO.getOrderDTO().getReceiptsTime())) {
                    // 超过评价时间
                    if (evaluate == null) {
                        resultMap.put(statusQO.getOrderId(), OrderReplyEnum.EVALUATE_OVERDUE.getCode());
                    } else {
                        resultMap.put(statusQO.getOrderId(), OrderReplyEnum.EVALUATE_HAS_OVERDUE.getCode());
                    }
                } else {
                    // 没有超过评价时间
                    if (evaluate != null) {
                        // 已经评价过
                        if (Constants.CLOSE.equals(evaluateOrderConfig.getReview())) {
                            // 判断追评是否支持
                            resultMap.put(statusQO.getOrderId(), OrderReplyEnum.REPLY_NOT_SUPPORT.getCode());
                        } else {
                            // 是否追评超过次数
                            List<TabEvaluateReply> replyList = this.evaluateReplyService.listEvaluateReply(evaluate.getEnterpriseId(), evaluate.getEvaluateId(), Constants.OPEN);
                            if (CollectionUtils.isNotEmpty(replyList) &&
                                    evaluateOrderConfig.getReviewTimes() <= this.evaluateReplyService.countReviewReplyByMember(enterpriseId, evaluate.getEvaluateId())) {
                                // 追评超过次数
                                resultMap.put(statusQO.getOrderId(), OrderReplyEnum.REPLY_OVER_MAX_TIMES.getCode());
                            } else {
                                //追评没有超过次数
                                if (Constants.OPEN.equals(evaluateOrderConfig.getBadEvaluateTag()) && Constants.OPEN.equals(evaluate.getMaliciousStatus())) {
                                    // 恶意评价
                                    if (evaluateOrderConfig.getBadEvaluateTagType() == -1 ||
                                            DateUtil.compareDate(DateUtil.addNumForHour(evaluate.getMaliciousStatusTime(), evaluateOrderConfig.getBadEvaluateTagType()), new Date())) {
                                        // 不能评价
                                        resultMap.put(statusQO.getOrderId(), OrderReplyEnum.EVALUATE_HAS_OVERDUE.getCode());
                                    }
                                } else {
                                    resultMap.put(statusQO.getOrderId(), evaluate.getReplyStatus());
                                }
                            }
                        }
                    } else {
                        //还没有评价过
                        resultMap.put(statusQO.getOrderId(), -1);
                    }
                }
            } else {
                resultMap.put(statusQO.getOrderId(), -1);
            }
        }
        return EnterpriseServiceResponse.success(resultMap);
    }

    @Override
    public ServiceResponse<Map<Integer, Integer>> queryEvaluateCountByStoreId(Integer enterpriseId, List<Integer> storeIdList) {
        Map<Integer, Integer> resultMap = new HashMap<>();
        Map<Integer, Map<String, Object>> evaluateCountByStoreIdMap = this.evaluateService.queryEvaluateCountByStoreId(enterpriseId, storeIdList);
        for (Integer storeId : storeIdList) {
            Map<String, Object> map = evaluateCountByStoreIdMap.get(storeId);
            resultMap.put(storeId, map != null ? Integer.valueOf(map.get("evaluateCount").toString()) : 0);
        }
        return EnterpriseServiceResponse.success(resultMap);
    }

    //    @Override
//    public ServiceResponse<Void> updateShowStatus(Integer enterpriseId, Integer evaluateReplyId, Long memberId, Integer showStatus) {
//        TabEvaluateOrderConfig evaluateOrderConfig = this.evaluateOrderConfigService.getEvaluateOrderConfig(enterpriseId);
//        if (Constants.OPEN.equals(evaluateOrderConfig.getReviewUserHidden())) {
//            this.evaluateReplyService.updateReplyShowStatus(enterpriseId, evaluateReplyId, showStatus);
//        }
//        return EnterpriseServiceResponse.success();
//    }

    private Integer isBadEvaluate(TabEvaluateOrderConfig evaluateOrderConfig, TabBadEvaluateConfig badEvaluateConfig, Integer goodsScore, Integer storeScore, Integer serviceScore) {
        // 任何一项评价达到设定值
        if (Constants.OPEN.equals(badEvaluateConfig.getBadOnceStatus())) {
            if (Constants.OPEN.equals(evaluateOrderConfig.getGoods()) && matchConfigValue(badEvaluateConfig.getBadGoodsType(), badEvaluateConfig.getBadGoodsScore(), goodsScore)) {
                // 商品质量
                return Constants.OPEN;
            }
            if (Constants.OPEN.equals(evaluateOrderConfig.getStore()) && matchConfigValue(badEvaluateConfig.getBadStoreType(), badEvaluateConfig.getBadStoreScore(), storeScore)) {
                // 门店服务
                return Constants.OPEN;
            }
            if (Constants.OPEN.equals(evaluateOrderConfig.getService()) && matchConfigValue(badEvaluateConfig.getBadServiceType(), badEvaluateConfig.getBadServiceScore(), serviceScore)) {
                // 服务能力
                return Constants.OPEN;
            }
        }
        // 三项评价均分

        if (Constants.OPEN.equals(badEvaluateConfig.getBadTotalStatus())) {
            // 开启的评价项
            int count = 0;
            // 评价总分
            int sumScore = 0;
            if (Constants.OPEN.equals(evaluateOrderConfig.getGoods())) {
                count++;
                sumScore += goodsScore;
            }
            if (Constants.OPEN.equals(evaluateOrderConfig.getStore())) {
                count++;
                sumScore += storeScore;
            }
            if (Constants.OPEN.equals(evaluateOrderConfig.getService())) {
                count++;
                sumScore += serviceScore;
            }

            if (matchConfigValue(badEvaluateConfig.getBadStoreType(), badEvaluateConfig.getBadStoreScore() * count, sumScore)) {
                return Constants.OPEN;
            }
        }
        return Constants.CLOSE;
    }

    private Integer isGoodEvaluate(TabEvaluateOrderConfig evaluateOrderConfig, TabBadEvaluateConfig badEvaluateConfig, Integer goodsScore, Integer storeScore, Integer serviceScore) {
        // 任何一项评价达到设定值
        if (Constants.OPEN.equals(badEvaluateConfig.getGoodStatus())) {
            // 三项评价均分

            // 开启的评价项
            int count = 0;
            // 评价总分
            int sumScore = 0;
            if (Constants.OPEN.equals(evaluateOrderConfig.getGoods())) {
                count++;
                sumScore += goodsScore;
            }
            if (Constants.OPEN.equals(evaluateOrderConfig.getStore())) {
                count++;
                sumScore += storeScore;
            }
            if (Constants.OPEN.equals(evaluateOrderConfig.getService())) {
                count++;
                sumScore += serviceScore;
            }

            if (matchConfigValue(badEvaluateConfig.getGoodTotalType(), badEvaluateConfig.getGoodTotalScore() * count, sumScore)) {
                if (Constants.OPEN.equals(evaluateOrderConfig.getGoods()) && matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), goodsScore)) {
                    // 商品质量
                    return Constants.OPEN;
                }
                if (Constants.OPEN.equals(evaluateOrderConfig.getStore()) && matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), storeScore)) {
                    // 门店服务
                    return Constants.OPEN;
                }
                if (Constants.OPEN.equals(evaluateOrderConfig.getService()) && matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), serviceScore)) {
                    // 服务能力
                    return Constants.OPEN;
                }
            }
        }
        return Constants.CLOSE;
    }

    private double getAvgScore(TabEvaluateOrderConfig evaluateOrderConfig, Integer goodsScore, Integer storeScore, Integer serviceScore) {
        logger.debug("计算平均分:{},{},{}", goodsScore, storeScore, serviceScore);
        // 开启的评价项
        double count = 0;
        // 评价总分
        double sumScore = 0;
        if (Constants.OPEN.equals(evaluateOrderConfig.getGoods()) && goodsScore != null) {
            count++;
            sumScore += goodsScore;
        }
        if (Constants.OPEN.equals(evaluateOrderConfig.getStore()) && storeScore != null) {
            count++;
            sumScore += storeScore;
        }
        if (Constants.OPEN.equals(evaluateOrderConfig.getService()) && serviceScore != null) {
            count++;
            sumScore += serviceScore;
        }
        if (count > 0) {
            double avg = sumScore / count;
            logger.debug("平均分计算结果：{}", avg);
            return avg;
        } else {
            logger.debug("平均分计算结果为0");
            return 0;
        }
    }

    private boolean matchConfigValue(Integer computeType, Integer computeScore, Integer score) {
        if (score == null) {
            return true;
        }
        switch (computeType) {
            case 1:
                return score < computeScore;
            case 2:
                return score <= computeScore;
            case 3:
                return score.equals(computeScore);
            case 4:
                return score >= computeScore;
            case 5:
                return score > computeScore;
            default:
                break;
        }
        return false;
    }

    private void saveQuick(String tagIds, Integer enterpriseId, Integer evaluateId, Long memberId) {
        if (StringUtils.isNotBlank(tagIds)) {
            String[] tagIdArr = tagIds.split(GlobalInfo.FLAG_COMMA);
            for (String tagId : tagIdArr) {
                evaluateTagsService.saveQuick(Integer.valueOf(tagId), enterpriseId, evaluateId, memberId);
            }
        }
    }

    private void saveQuick(String tagIds, Integer enterpriseId, Integer evaluateId, Integer evaluateReplyId, Long memberId) {
        if (StringUtils.isNotBlank(tagIds)) {
            String[] tagIdArr = tagIds.split(GlobalInfo.FLAG_COMMA);
            for (String tagId : tagIdArr) {
                evaluateTagsService.saveQuick(Integer.valueOf(tagId), enterpriseId, evaluateId, evaluateReplyId, memberId);
            }
        }
    }

}
