package com.gic.evaluate.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.data.api.dto.StoreOrderEvaluateDTO;
import com.gic.data.api.service.StoreEvaluateApiService;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.QuickReplyTypeEnum;
import com.gic.evaluate.qo.StoreScoreQO;
import com.gic.evaluate.qo.StoreTrendQO;
import com.gic.evaluate.service.EvaluateOverviewApiService;
import com.gic.evaluate.service.EvaluateQuickService;
import com.gic.evaluate.service.EvaluateService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 16:52
 */
@SuppressWarnings("ALL")
@Service("evaluateOverviewApiService")
public class EvaluateOverviewApiServiceImpl implements EvaluateOverviewApiService {

    private Logger logger = LogManager.getLogger(EvaluateOverviewApiServiceImpl.class);

    @Autowired
    private EvaluateService evaluateService;
    @Autowired
    private EvaluateQuickService evaluateQuickService;
    @Autowired
    StoreEvaluateApiService storeEvaluateApiService;

    @Override
    public ServiceResponse<Map<String, Long>> chart1(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        Map<String, Long> countMap = evaluateService.getTotalCountChart(enterpriseId, startTime, endTime, storeIdList);
        return EnterpriseServiceResponse.success(countMap);
    }

    @Override
    public ServiceResponse<HashMap<String, Map<String, Long>>> chart2(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        Map<String, Long> goodsCountMap = evaluateService.getGoodsCountChart(enterpriseId, startTime, endTime, storeIdList);
        Map<String, Long> storeCountMap = evaluateService.getStoreCountChart(enterpriseId, startTime, endTime, storeIdList);
        Map<String, Long> serviceCountMap = evaluateService.getServiceCountChart(enterpriseId, startTime, endTime, storeIdList);

        HashMap<String, Map<String, Long>> hashMap = new HashMap<>(4);
        hashMap.put("goodsCountMap", goodsCountMap);
        hashMap.put("storeCountMap", storeCountMap);
        hashMap.put("serviceCountMap", serviceCountMap);


        return EnterpriseServiceResponse.success(hashMap);
    }

    @Override
    public ServiceResponse<Map<String, List<Map<String, Object>>>> chart3(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        List<Map<String, Object>> listTrendTotalEvaluate = this.evaluateService.listTrendTotalEvaluate(enterpriseId, startTime, endTime, storeIdList);
        List<Map<String, Object>> listTrendGoodEvaluate = this.evaluateService.listTrendGoodEvaluate(enterpriseId, startTime, endTime, storeIdList);
        List<Map<String, Object>> listTrendOrderCount = this.storeEvaluateApiService.listTrendOrderCount(enterpriseId, startTime, endTime, storeIdList).getResult();
        Map<String, List<Map<String, Object>>> map = new HashMap<>();


        try {
            List<String> dateList = this.getDateList(startTime, endTime);

            map.put("totalEvaluateList", dealChart4(listTrendTotalEvaluate, dateList));
            map.put("goodEvaluateList", dealChart4(listTrendGoodEvaluate, dateList));
            map.put("orderList", dealChart4(listTrendOrderCount, dateList));
            return EnterpriseServiceResponse.success(map);
        } catch (Exception e) {
            logger.info("解析参数错误", e);
            return EnterpriseServiceResponse.failure(ErrorCode.SYSTEM_ERROR);
        }
    }

    @Override
    public ServiceResponse<Map<String, List<Map<String, Object>>>> chart4(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        List<Map<String, Object>> goodChart = this.evaluateQuickService.evaluateGoodQuickChart(enterpriseId, startTime, endTime, storeIdList);
        List<Map<String, Object>> badChart = this.evaluateQuickService.evaluateBadQuickChart(enterpriseId, startTime, endTime, storeIdList);

        Map<String, List<Map<String, Object>>> map = new HashMap<>(4);
        map.put("goodChart", goodChart);
        map.put("badChart", badChart);
        return EnterpriseServiceResponse.success(map);
    }

    private List<Map<String, Object>> dealChart4(List<Map<String, Object>> chart, List<String> dateList) {
        List<Map<String, Object>> result = new ArrayList<>();
        for (String date : dateList) {
            Map<Object, Object> collect = chart.stream().collect(Collectors.toMap(e -> e.get("statisticsDate"), e -> e.get("count")));
            Map<String, Object> map = new HashMap<>(4);
            map.put(date, collect.getOrDefault(date, 0));
            result.add(map);
        }
        return result;
    }

    @Override
    public ServiceResponse<Page<Map<String, Object>>> listStoreScore(StoreScoreQO storeScoreQO) {
        com.github.pagehelper.Page<Map<String, Object>> storeScoreList = this.evaluateService.listStoreScore(storeScoreQO);
        if (CollectionUtils.isNotEmpty(storeScoreList)) {
            Map<String, Object> storeScore = this.evaluateService.getTotalStoreScore(storeScoreQO);
            storeScore.put("storeName", "总计");
            storeScoreList.add(0, storeScore);
        }
        return EnterpriseServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(storeScoreList));
    }

    @Override
    public ServiceResponse<Map<String, Object>> getTotalStoreScore(StoreScoreQO storeScoreQO) {
        Map<String, Object> storeScore = this.evaluateService.getTotalStoreScore(storeScoreQO);
        return EnterpriseServiceResponse.success(storeScore);
    }

    @Override
    public ServiceResponse<Page<Map<String, Object>>> listTrend(StoreTrendQO storeTrendQO) {
        com.github.pagehelper.Page<Map<String, Object>> trendList = this.evaluateService.listTrend(storeTrendQO);
        if (CollectionUtils.isNotEmpty(trendList)) {
            List<Integer> storeIdList = trendList.stream().map(
                    e -> {
//                        e.put("evaluateRate", 0);
                        Object storeId = e.get("storeId");
                        return storeId == null ? null : Integer.valueOf(storeId.toString());
                    }
            ).collect(Collectors.toList());
            // 这里要取数据组的数据
            Map<Integer, StoreOrderEvaluateDTO> orderCountMap = storeEvaluateApiService.getStoreOrderEvaluateByStoreId(storeTrendQO.getEnterpriseId(), storeIdList).getResult();
            for (Map<String, Object> e : trendList) {
                Object storeId = e.get("storeId");
                if (storeId != null) {
                    StoreOrderEvaluateDTO storeOrderEvaluateDTO = orderCountMap.get(Integer.valueOf(storeId.toString()));
                    if (storeOrderEvaluateDTO != null) {
                        BigDecimal goodRate = (BigDecimal) e.get("goodRate");
                        e.put("evaluateRate", storeOrderEvaluateDTO.getEvaluateRate());

                        if (goodRate != null) {
                            e.put("goodRate", (double)Math.round(goodRate.doubleValue() * 10000) / 100 + "%");
                        } else {
                            e.put("goodRate", 0);
                        }
                    } else {
                        e.put("evaluateRate", 0);
                    }
                }

            }
        }
        return EnterpriseServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(trendList));
    }

    @Override
    public ServiceResponse<List<Map<String, Integer>>> listGoodEvaluateTrendByDate(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        // todo 放假回来改成循环
        return getListServiceResponse(enterpriseId, startTime, endTime, QuickReplyTypeEnum.GOOD, storeIdList);
    }

    @Override
    public ServiceResponse<List<Map<String, Integer>>> listBadEvaluateTrendByDate(Integer enterpriseId, String startTime, String endTime, List<Integer> storeIdList) {
        // todo 放假回来改成循环
        return getListServiceResponse(enterpriseId, startTime, endTime, QuickReplyTypeEnum.BAD, storeIdList);
    }

    private ServiceResponse<List<Map<String, Integer>>> getListServiceResponse(Integer enterpriseId, String startTime, String endTime, QuickReplyTypeEnum quickReplyTypeEnum, List<Integer> storeIdList) {
        Date[] time = getTime(startTime, endTime);
        Integer count1 = this.evaluateService.countEvaluateTrend(enterpriseId, time[0], time[1], storeIdList, quickReplyTypeEnum.getCode());
        Integer count2 = this.evaluateService.countEvaluateTrend(enterpriseId, time[1], time[2], storeIdList, quickReplyTypeEnum.getCode());
        Integer count3 = this.evaluateService.countEvaluateTrend(enterpriseId, time[2], time[3], storeIdList, quickReplyTypeEnum.getCode());
        Integer count4 = this.evaluateService.countEvaluateTrend(enterpriseId, time[3], time[4], storeIdList, quickReplyTypeEnum.getCode());
        Integer count5 = this.evaluateService.countEvaluateTrend(enterpriseId, time[4], null, storeIdList, quickReplyTypeEnum.getCode());
        Map<String, Integer> map1 = new HashMap<>(4);
        map1.put(DateUtil.dateToStr(time[0], DateUtil.FORMAT_DATE_10), count1);
        Map<String, Integer> map2 = new HashMap<>(4);
        map2.put(DateUtil.dateToStr(time[1], DateUtil.FORMAT_DATE_10), count2);
        Map<String, Integer> map3 = new HashMap<>(4);
        map3.put(DateUtil.dateToStr(time[2], DateUtil.FORMAT_DATE_10), count3);
        Map<String, Integer> map4 = new HashMap<>(4);
        map4.put(DateUtil.dateToStr(time[3], DateUtil.FORMAT_DATE_10), count4);
        Map<String, Integer> map5 = new HashMap<>(4);
        map5.put(DateUtil.dateToStr(time[4], DateUtil.FORMAT_DATE_10), count5);

        List<Map<String, Integer>> list = new ArrayList<>();
        list.add(map1);
        list.add(map2);
        list.add(map3);
        list.add(map4);
        list.add(map5);
        return EnterpriseServiceResponse.success(list);
    }

    private Date[] getTime(String startTime, String endTime) {
        Date[] arr = new Date[5];
//        String startTime = "2019-12-01";
//        String endTime = "2020-01-17";
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date startTimeDate;
        Date endTimeDate;
        try {
            startTimeDate = sdf.parse(startTime);
            endTimeDate = sdf.parse(endTime);
            Calendar cal1 = Calendar.getInstance();
            cal1.setTime(startTimeDate);
            Date date1 = cal1.getTime();
            long time1 = cal1.getTimeInMillis();
            Calendar cal2 = Calendar.getInstance();
            cal2.setTime(endTimeDate);
            long time2 = cal2.getTimeInMillis();
            long betweenDays = (time2 - time1) / 86400000L;
            int days = Integer.parseInt(String.valueOf(betweenDays));
            int step = days/5;

            cal1.add(Calendar.DAY_OF_MONTH, step);
            Date date2 = cal1.getTime();
            cal1.add(Calendar.DAY_OF_MONTH, step);
            Date date3 = cal1.getTime();
            cal1.add(Calendar.DAY_OF_MONTH, step);
            Date date4 = cal1.getTime();
            cal1.add(Calendar.DAY_OF_MONTH, step);
            Date date5 = cal1.getTime();
            arr[0] = date1;
            arr[1] = date2;
            arr[2] = date3;
            arr[3] = date4;
            arr[4] = date5;
        } catch (ParseException var10) {
            logger.warn("exception", var10);
        }
        return arr;

    }

    private List<String> getDateList(String dateFirst, String dateSecond) throws ParseException {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date dateOne = dateFormat.parse(dateFirst);
        Date dateTwo = dateFormat.parse(dateSecond);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateOne);

        List<String> dateList = new ArrayList<>();
        while (calendar.getTime().before(dateTwo)) {
            dateList.add(dateFormat.format(calendar.getTime()));
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }
        dateList.add(dateSecond);
        return dateList;
    }
}
