package com.gic.evaluate.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.constant.ReplyerTypeEnum;
import com.gic.evaluate.dto.ProblemDTO;
import com.gic.evaluate.dto.ProblemReplyDTO;
import com.gic.evaluate.dto.ProblemUserReplyDTO;
import com.gic.evaluate.entity.TabProblem;
import com.gic.evaluate.entity.TabProblemImg;
import com.gic.evaluate.qo.ProblemQO;
import com.gic.evaluate.service.ProblemImgService;
import com.gic.evaluate.service.ProblemManageApiService;
import com.gic.evaluate.service.ProblemReplyService;
import com.gic.evaluate.service.ProblemService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.shaded.com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-02 11:08
 */
@Service("problemManageApiService")
public class ProblemManageApiServiceImpl implements ProblemManageApiService {

    @Autowired
    private ProblemService problemService;
    @Autowired
    private ProblemImgService problemImgService;
    @Autowired
    private ProblemReplyService problemReplyService;

    @Override
    public ServiceResponse<Page<ProblemDTO>> listProblem(ProblemQO problemQO) {
        com.github.pagehelper.Page<TabProblem> problemList = problemService.listProblem(problemQO);
        Page<ProblemDTO> problemDtoPage = PageHelperUtils.changePageHelperToCurrentPage(problemList, ProblemDTO.class);
        return EnterpriseServiceResponse.success(problemDtoPage);
    }

    @Override
    public ServiceResponse<Integer> countProblem(ProblemQO problemQO) {
        Integer count = problemService.countProblem(problemQO);
        return EnterpriseServiceResponse.success(count);
    }

    @Override
    public ServiceResponse<ProblemDTO> getProblem(Integer problemId) {
        TabProblem tabProblem = problemService.getProblem(problemId);
        if (tabProblem == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        ProblemDTO problemDTO = EntityUtil.changeEntityByJSON(ProblemDTO.class, tabProblem);
        if (Constants.OPEN.equals(problemDTO.getHasOtherImage())) {
            List<TabProblemImg> tabImgList = problemImgService.listImageByProblemId(problemId);
            List<String> list = Optional.ofNullable(tabImgList).orElse(Collections.emptyList()).stream().map(TabProblemImg::getImgUrl).collect(Collectors.toList());
            problemDTO.setImgList(list);
        } else if (StringUtils.isNotBlank(problemDTO.getImageUrl())) {
            problemDTO.setImgList(Lists.newArrayList(problemDTO.getImageUrl()));
        }
        List<ProblemReplyDTO> replyList = problemReplyService.listReply(problemId);

        if (CollectionUtils.isNotEmpty(replyList)) {
            for (ProblemReplyDTO replyDTO : replyList) {
                List<TabProblemImg> imgList = this.problemImgService.listImageByProblemReplyId(replyDTO.getProblemReplyId());
                replyDTO.setImgList(Optional.ofNullable(imgList).orElse(Collections.emptyList()).stream().map(TabProblemImg::getImgUrl).collect(Collectors.toList()));
            }
        }
        ProblemReplyDTO problemReplyDTO = new ProblemReplyDTO();
        problemReplyDTO.setReplyerType(ReplyerTypeEnum.MEMBER.getCode());
        problemReplyDTO.setReplyContent(tabProblem.getProblemContent());
        problemReplyDTO.setMemberId(tabProblem.getMemberId());
        problemReplyDTO.setMemberName(tabProblem.getMemberName());
        problemReplyDTO.setMemberImgUrl(tabProblem.getMemberImgUrl());
        problemReplyDTO.setImgList(problemDTO.getImgList());
        problemReplyDTO.setCreateTime(problemDTO.getCreateTime());
        problemReplyDTO.setUpdateTime(problemDTO.getUpdateTime());
        if (replyList == null) {
            replyList = new ArrayList<>();
        } else {
            replyList = new ArrayList<>(replyList);
        }
        replyList.add(0, problemReplyDTO);

        problemDTO.setReplyList(replyList);
        return EnterpriseServiceResponse.success(problemDTO);
    }

    @Override
    public ServiceResponse<Void> replyProblem(ProblemUserReplyDTO problemUserReplyDTO) {
        this.problemReplyService.save(problemUserReplyDTO);
        ProblemDTO problemDTO = new ProblemDTO();
        problemDTO.setProblemId(problemUserReplyDTO.getProblemId());
        problemDTO.setProblemStatus(1);
        this.problemService.updateProblem(problemDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> replyProblem(List<Integer> problemIdList, ProblemUserReplyDTO problemUserReplyDTO) {
        for (Integer problemId : problemIdList) {
            problemUserReplyDTO.setProblemId(problemId);
            this.replyProblem(problemUserReplyDTO);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateMaliciousStatus(Integer problemId, Integer maliciousStatus) {
        ProblemDTO problemDTO = new ProblemDTO();
        problemDTO.setProblemId(problemId);
        problemDTO.setMaliciousStatus(maliciousStatus);
        if (Constants.OPEN.equals(maliciousStatus)) {
            problemDTO.setShowStatus(Constants.CLOSE);
            problemDTO.setMaliciousStatusTime(new Date());
        } else if (Constants.CLOSE.equals(maliciousStatus)) {
            problemDTO.setShowStatus(Constants.OPEN);
        }
        problemService.updateProblem(problemDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> deleteReply(Integer problemReplyId) {
        ProblemReplyDTO problemReplyDTO = new ProblemReplyDTO();
        problemReplyDTO.setProblemReplyId(problemReplyId);
        problemReplyDTO.setStatus(Constants.CLOSE);
        problemReplyService.update(problemReplyDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> editReplyProblem(ProblemUserReplyDTO problemUserReplyDTO) {
        ProblemReplyDTO problemReplyDTO = new ProblemReplyDTO();
        problemReplyDTO.setProblemReplyId(problemUserReplyDTO.getProblemReplyId());
        problemReplyDTO.setReplyContent(problemUserReplyDTO.getReplyContent());
        problemReplyService.update(problemReplyDTO);
        return EnterpriseServiceResponse.success();
    }
}
