package com.gic.evaluate.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.dto.ProblemDTO;
import com.gic.evaluate.dto.ProblemMemberReplyDTO;
import com.gic.evaluate.dto.ProblemReplyDTO;
import com.gic.evaluate.entity.TabEvaluateOrderConfig;
import com.gic.evaluate.entity.TabProblem;
import com.gic.evaluate.entity.TabProblemImg;
import com.gic.evaluate.service.*;
import com.gic.member.api.dto.AcuDetailDTO;
import com.gic.member.api.dto.MemberUserDTO;
import com.gic.member.api.service.CuDetailService;
import com.gic.member.api.service.MemberUserApiService;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.StoreApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-02 14:31
 */
@Service("problemOutApiService")
public class ProblemOutApiServiceImpl implements ProblemOutApiService {

    private Logger logger = LogManager.getLogger(ProblemOutApiServiceImpl.class);

    @Autowired
    private ProblemService problemService;
    @Autowired
    private ProblemReplyService problemReplyService;
    @Autowired
    private ProblemImgService problemImgService;
    @Autowired
    private MemberUserApiService memberUserApiService;
    @Autowired
    private EvaluateOrderConfigService evaluateOrderConfigService;
    @Autowired
    private CuDetailService cuDetailService;
    @Autowired
    private StoreApiService storeApiService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> saveProblem(ProblemDTO problemDTO) {
        logger.info("保存咨询:{}", JSON.toJSONString(problemDTO));
        List<String> imgList = problemDTO.getImgList();
        if (CollectionUtils.isNotEmpty(imgList)) {
            problemDTO.setImageUrl(imgList.get(0));
            problemDTO.setHasOtherImage(imgList.size() > 1 ? Constants.OPEN : Constants.CLOSE);
        }

        ServiceResponse<MemberUserDTO> memberUserDtoServiceResponse = memberUserApiService.getByMcuId(problemDTO.getMemberId(), problemDTO.getEnterpriseId());
        if (memberUserDtoServiceResponse.isSuccess()) {
            MemberUserDTO memberUserDTO = memberUserDtoServiceResponse.getResult();
            problemDTO.setMemberName(memberUserDTO.getName());
            problemDTO.setMemberPhone(memberUserDTO.getMobile());
            // 门店
            ServiceResponse<String> serviceStoreId = cuDetailService.getServiceStoreId(problemDTO.getAreaId(), problemDTO.getMemberId(), problemDTO.getUnionId(), problemDTO.getEnterpriseId(), problemDTO.getEcuId());

            if (serviceStoreId != null && StringUtils.isNotBlank(serviceStoreId.getResult())) {
                StoreDTO storeDTO = storeApiService.getStoreByStoreInfoId(memberUserDTO.getEnterpriseId(), Integer.valueOf(serviceStoreId.getResult())).getResult();
                if (storeDTO != null) {
                    problemDTO.setStoreName(storeDTO.getStoreName());
                }
            }

            if (problemDTO.getAcuId() != null) {
                ServiceResponse<AcuDetailDTO> acuDetailResponse = cuDetailService.getAcuDetail(problemDTO.getAcuId(), problemDTO.getEnterpriseId(), problemDTO.getEcuId());
                if (acuDetailResponse.isSuccess() || acuDetailResponse.getResult() != null) {
                    problemDTO.setMemberImgUrl(acuDetailResponse.getResult().getHeadUrl());
                }
            }
        }
        Integer problemId = problemService.save(problemDTO);
        if (CollectionUtils.isNotEmpty(imgList)) {
            problemImgService.saveList(problemDTO.getEnterpriseId(), problemId, imgList);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveProblemReply(ProblemMemberReplyDTO problemMemberReplyDTO) {
        logger.info("保存咨询追评:{}", JSON.toJSONString(problemMemberReplyDTO));

        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(problemMemberReplyDTO.getEnterpriseId());
        TabProblem problem = this.problemService.getProblem(problemMemberReplyDTO.getProblemId());
        if (problem == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        if (Constants.OPEN.equals(evaluateOrderConfig.getBadSuggestionTag())) {
            if (Constants.OPEN.equals(problem.getMaliciousStatus())) {
                Date date = DateUtil.addNumForHour(new Date(), -evaluateOrderConfig.getBadSuggestionTagType());
                if (DateUtil.compareDate(problem.getMaliciousStatusTime(), date)) {
                    return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "恶评状态下不能评论");
                }
            }
        }

        List<String> imgUrlList = problemMemberReplyDTO.getImgList();
        if (problemMemberReplyDTO.getMemberId() != null) {
            ServiceResponse<MemberUserDTO> memberUserDtoServiceResponse = memberUserApiService.getByMcuId(problemMemberReplyDTO.getMemberId(), problemMemberReplyDTO.getEnterpriseId());
            if (memberUserDtoServiceResponse.isSuccess()) {
                MemberUserDTO memberUserDTO = memberUserDtoServiceResponse.getResult();
                problemMemberReplyDTO.setMemberName(memberUserDTO.getName());
            }
        }
        if (problemMemberReplyDTO.getAcuId() != null) {
            ServiceResponse<AcuDetailDTO> acuDetailResponse = cuDetailService.getAcuDetail(problemMemberReplyDTO.getAcuId(), problemMemberReplyDTO.getEnterpriseId(), problemMemberReplyDTO.getEcuId());
            if (acuDetailResponse.isSuccess() || acuDetailResponse.getResult() != null) {
                problemMemberReplyDTO.setMemberImgUrl(acuDetailResponse.getResult().getHeadUrl());
            }
        }
        Integer problemReplyId = problemReplyService.save(problemMemberReplyDTO);
        ProblemDTO problemDTO = new ProblemDTO();
        problemDTO.setProblemId(problemMemberReplyDTO.getProblemId());
        problemDTO.setProblemStatus(2);
        this.problemService.updateProblem(problemDTO);
        if (CollectionUtils.isNotEmpty(imgUrlList)) {
            problemImgService.saveList(problemMemberReplyDTO.getEnterpriseId(), problemMemberReplyDTO.getProblemId(), problemReplyId, imgUrlList);
        }

        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<ProblemDTO>> listProblem(Long acuId) {
        List<ProblemDTO> problemList = problemService.listProblem(acuId);
        if (CollectionUtils.isNotEmpty(problemList)) {
            TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(problemList.get(0).getEnterpriseId());
            List<Integer> problemIdList = problemList.stream().map(ProblemDTO::getProblemId).collect(Collectors.toList());
            Map<Integer, String> firstReply = problemReplyService.firstReplyByProblemId(problemIdList);

            for (ProblemDTO problemDTO : problemList) {
                problemDTO.setProblemCount(problemDTO.getProblemCount() == null ? 1 : (problemDTO.getProblemCount() + 1));
                problemDTO.setReplyContent(firstReply.get(problemDTO.getProblemId()));

                if (Constants.OPEN.equals(evaluateOrderConfig.getBadSuggestionTag())) {
                    if (Constants.OPEN.equals(problemDTO.getMaliciousStatus())) {
                        Date date = DateUtil.addNumForHour(new Date(), -evaluateOrderConfig.getBadSuggestionTagType());
                        if (DateUtil.compareDate(problemDTO.getMaliciousStatusTime(), date)) {
                            problemDTO.setStatus(2);
                        }
                    }
                }

                List<String> imgList = Optional.ofNullable(problemImgService.listImageByProblemId(problemDTO.getProblemId())).orElse(Collections.emptyList())
                        .stream().map(TabProblemImg::getImgUrl).collect(Collectors.toList());
                problemDTO.setImgList(imgList);

            }
        }
        return EnterpriseServiceResponse.success(problemList);
    }

    @Override
    public ServiceResponse<ProblemDTO> getProblem(Integer problemId) {
        TabProblem problem = problemService.getProblem(problemId);
        ProblemDTO problemDTO = EntityUtil.changeEntityByJSON(ProblemDTO.class, problem);
        List<TabProblemImg> imgList = problemImgService.listImageByProblemId(problemId);
        List<String> list = Optional.ofNullable(imgList).orElse(Collections.emptyList()).stream().map(TabProblemImg::getImgUrl).collect(Collectors.toList());
        problemDTO.setImgList(list);
        List<ProblemReplyDTO> replyList = problemReplyService.listReply(problemId);
        for (ProblemReplyDTO problemReplyDTO : replyList) {
            List<TabProblemImg> tabProblemImgs = problemImgService.listImageByProblemReplyId(problemReplyDTO.getProblemReplyId());
            List<String> replyImgList = Optional.ofNullable(tabProblemImgs).orElse(Collections.emptyList()).stream().map(TabProblemImg::getImgUrl).collect(Collectors.toList());
            problemReplyDTO.setImgList(replyImgList);
        }
        problemDTO.setReplyList(replyList);
        return EnterpriseServiceResponse.success(problemDTO);
    }

    @Override
    public ServiceResponse<Void> endProblem(Integer problemId) {
        problemService.endProblem(problemId);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> ensure(Integer enterpriseId, Long acuId) {
        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(enterpriseId);
        if (Constants.CLOSE.equals(evaluateOrderConfig.getBadSuggestionTag())) {
            return EnterpriseServiceResponse.success();
        }
        boolean b = problemService.hasMaliciousStatusByAcuIdBeforeHours(acuId, evaluateOrderConfig.getBadSuggestionTagType());
        if (b) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "已被标记为恶意投诉，暂不可投诉");
        }
        return EnterpriseServiceResponse.success();
    }
}