package com.gic.evaluate.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.DataDownloadUtils;
import com.gic.download.utils.DownloadReport;
import com.gic.download.utils.DownloadUtils;
import com.gic.download.utils.ExecutorPoolSingleton;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.evaluate.constant.ReplyerTypeEnum;
import com.gic.evaluate.dto.EvaluateDTO;
import com.gic.evaluate.dto.EvaluateReplyDTO;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.qo.ExportQO;
import com.gic.evaluate.service.EvaluateManageApiService;
import com.gic.evaluate.web.utils.DataContentUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-14 16:47
 */
@RestController
public class EvaluateExportController extends DownloadUtils {

    @Autowired
    private EvaluateManageApiService evaluateManageApiService;

    private Logger logger = LogManager.getLogger(EvaluateExportController.class);

    @RequestMapping("/list-evaluate-field")
    public RestResponse listEvaluateField() {
        EvaluateFieldEnum[] values = EvaluateFieldEnum.values();
        List<Map<String, Object>> arrayList = new ArrayList<>(values.length);
        for (EvaluateFieldEnum value : values) {
            Map<String, Object> hashMap = new HashMap<>(4);
            hashMap.put("fieldCode", value.getFieldCode());
            hashMap.put("fieldName", value.getFieldName());
            hashMap.put("needEncrypt", value.getNeedEncrypt());
            arrayList.add(hashMap);
        }
        return RestResponse.success(arrayList);
    }

    @RequestMapping("/export-evaluate")
    public RestResponse exportEvaluateRecord(HttpServletRequest request, EvaluateQO evaluateQO, ExportQO exportQO) {
        evaluateQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (evaluateQO.getEvaluateIds() != null && evaluateQO.getEvaluateIds().length == 1 && evaluateQO.getEvaluateIds()[0] == -1) {
            // 查询全部
            evaluateQO.setEvaluateIds(null);
        }
        ServiceResponse<Integer> serviceResponse = evaluateManageApiService.countEvaluate(evaluateQO);

        List<String> fieldCodeList = exportQO.getFieldCodeList();
        List<String> fieldNameList = new ArrayList<>(fieldCodeList.size());
        for (String fieldCode: fieldCodeList) {
            String fieldName = EvaluateFieldEnum.valueOf(fieldCode).getFieldName();
            fieldNameList.add(fieldName);
        }
        // 开始下载
        String dataContent = DataContentUtils.getDataContent();
        // 生成入参数据
        DownloadReport report = new DownloadReport();
        //申请人ID
        report.setApplyUserId(UserDetailUtils.getUserDetail().getUserId());
        //数据源（频道页），用  "-" 隔开上级和本级
        report.setDataContent(dataContent);
        //待下载数据的数据总量
        report.setDataCount(serviceResponse.getResult());
        //报告类型  1：完整数据， 2：脱敏数据， 3：二维码
        report.setDataType(exportQO.getDataType());
        //数据源超链接，用于点击跳转对应频道页面
        report.setDataUrl("-");
        report.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //待下载文件名
        report.setFileName(exportQO.getFileName());
        //待下载数据的结束时间
        if (StringUtils.isNotBlank(evaluateQO.getEndTime())) {
            report.setReportEndTime(DateUtil.strToDate(evaluateQO.getEndTime(), DateUtil.FORMAT_DATETIME_19));
        }
        //待下载数据的开始时间
        if (StringUtils.isNotBlank(evaluateQO.getStartTime())) {
            report.setReportStartTime(DateUtil.strToDate(evaluateQO.getStartTime(), DateUtil.FORMAT_DATETIME_19));
        }
        //调用生成方法，获取reportId
        Integer reportId = DataDownloadUtils.createDownloadReport(report);
        String path = request.getSession().getServletContext().getRealPath("/excel/csv/collaborator/file/");
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(DateUtil.FORMAT_DATETIME_19);
        ExecutorPoolSingleton.getInstance().executeTask(() -> {
            DownloadDataLoader<EvaluateDTO> loader = new DownloadDataLoader<EvaluateDTO>() {
                @Override
                protected List<EvaluateDTO> getDownloadData(Integer pageNum) throws Exception {
                    //调用分页查询接口
                    evaluateQO.setCurrentPage(pageNum);
                    evaluateQO.setPageSize(1000);
                    ServiceResponse<Page<EvaluateDTO>> result = evaluateManageApiService.listEvaluateForManage(evaluateQO);
                    // todo 这里还要处理特殊数据
                    boolean hasReplyTime = fieldCodeList.contains("replyTime");
                    boolean hasReplyMemberContent = fieldCodeList.contains("replyMemberContent");
                    boolean hasReplyUserContent = fieldCodeList.contains("replyUserContent");
                    if (CollectionUtils.isNotEmpty(result.getResult().getResult())) {
                        parse(result, hasReplyTime, hasReplyMemberContent, hasReplyUserContent, simpleDateFormat);
                    }

                    return result.getResult().getResult();
                }
            };
            try {
                //参数说明：excelExtensionCode，Excel后缀名code ，枚举类：ExcelExtensionEnum, Arrays.asList("phone")这个phone是需要加密的字段
                download(path, reportId, exportQO.getFileName(), exportQO.getExcelExtension(), fieldNameList, fieldCodeList, loader, Arrays.asList("memberPhone"), null);
            } catch (Exception e) {
                logger.warn("下载错误", e);
            }
        });
        return RestResponse.success(reportId);
    }

    private void parse(ServiceResponse<Page<EvaluateDTO>> result, boolean hasReplyTime, boolean hasReplyMemberContent, boolean hasReplyUserContent, SimpleDateFormat simpleDateFormat) {
        for(EvaluateDTO resultDTO : result.getResult().getResult()) {
            EvaluateDTO evaluateDTO = null;
            if (hasReplyTime || hasReplyMemberContent || hasReplyUserContent) {
                evaluateDTO = evaluateManageApiService.getEvaluate(resultDTO.getEnterpriseId(), resultDTO.getEvaluateId()).getResult();
            }
            // 追评时间
            if (hasReplyTime) {
                List<EvaluateReplyDTO> replyList = evaluateDTO.getReplyList();
                List<String> replyDateList = Optional.ofNullable(replyList).orElse(Collections.emptyList()).stream()
                        .filter(e -> ReplyerTypeEnum.MEMBER.getCode().equals(e.getReplyerType()))
                        .map(e -> simpleDateFormat.format(e.getReplyDate()))
                        .collect(Collectors.toList());
                StringBuilder sb = new StringBuilder();
                for (int i = 0; i < replyDateList.size(); i++) {
                    sb.append("(").append(i + 1).append(")").append(replyDateList.get(i)).append("\r\n");
                }
                resultDTO.setReplyTime(sb.toString());
            }
            // 追评内容
            if (hasReplyMemberContent) {
                List<EvaluateReplyDTO> replyList = evaluateDTO.getReplyList();
                List<String> replyMemberContentList = Optional.ofNullable(replyList).orElse(Collections.emptyList()).stream()
                        .filter(e -> ReplyerTypeEnum.MEMBER.getCode().equals(e.getReplyerType()))
                        .map(EvaluateReplyDTO::getReplyContent)
                        .collect(Collectors.toList());
                StringBuilder sb = new StringBuilder();
                for (int i = 0; i < replyMemberContentList.size(); i++) {
                    sb.append("(").append(i + 1).append(")").append(replyMemberContentList.get(i)).append("\r\n");
                }
                resultDTO.setReplyMemberContent(sb.toString());
            }
            // 商家回复
            if (hasReplyUserContent) {
                List<EvaluateReplyDTO> replyList = evaluateDTO.getReplyList();
                List<String> replyUserContentList = Optional.ofNullable(replyList).orElse(Collections.emptyList()).stream()
                        .filter(e -> ReplyerTypeEnum.USER.getCode().equals(e.getReplyerType()))
                        .map(EvaluateReplyDTO::getReplyContent)
                        .collect(Collectors.toList());
                StringBuilder sb = new StringBuilder();
                for (int i = 0; i < replyUserContentList.size(); i++) {
                    sb.append("(").append(i + 1).append(")").append(replyUserContentList.get(i)).append("\r\n");
                }
                resultDTO.setReplyUserContent(sb.toString());
            }
        }
    }

    private enum EvaluateFieldEnum{
        /**
         * 首次评价时间
         */
        evaluateTime("evaluateTime", "首次评价时间", 0),
        /**
         * 会员姓名
         */
        memberName("memberName", "会员姓名", 0),
        /**
         * 会员手机号
         */
        memberPhone("memberPhone", "会员手机号", 1),
        /**
         * 商品质量评分
         */
        goodsScore("goodsScore", "商品质量评分", 0),
        /**
         * 门店环境评分
         */
        storeScore("storeScore", "门店环境评分", 0),
        /**
         * 服务能力评分
         */
        serviceScore("serviceScore", "服务能力评分", 0),
        /**
         * 评语
         */
        evaluateContent("evaluateContent", "评语", 0),
        /**
         * 追评时间
         */
        replyTime("replyTime", "追评时间", 0),
        /**
         * 追评内容
         */
        replyMemberContent("replyMemberContent", "追评内容", 0),
        /**
         * 商家回复
         */
        replyUserContent("replyUserContent", "商家回复", 0),
        /**
         * 下单时间
         */
        receiptsDate("receiptsDate", "下单时间", 0),
        /**
         * 订单号
         */
        orderNumber("orderNumber", "订单号", 0),
        /**
         * 订单金额
         */
        orderMount("orderMount", "订单金额", 0),
        /**
         * 商品信息
         */
        goodsInfo("goodsInfo", "商品信息", 0),
        /**
         * 门店名称
         */
        storeName("storeName", "门店名称", 0),
        /**
         * 门店电话
         */
        storePhone("storePhone", "门店电话", 0),
        ;
        private String fieldCode;
        private String fieldName;
        private Integer needEncrypt;

        EvaluateFieldEnum(String fieldCode, String fieldName, Integer needEncrypt) {
            this.fieldCode = fieldCode;
            this.fieldName = fieldName;
            this.needEncrypt = needEncrypt;
        }

        public String getFieldCode() {
            return fieldCode;
        }

        public String getFieldName() {
            return fieldName;
        }

        public Integer getNeedEncrypt() {
            return needEncrypt;
        }
    }

}