package com.gic.evaluate.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.ResourceHoldEnum;
import com.gic.auth.dto.OrderResourceDTO;
import com.gic.auth.service.ResourceApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.dto.EvaluateDTO;
import com.gic.evaluate.dto.EvaluateQuickDTO;
import com.gic.evaluate.dto.UserReplyDTO;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.service.EvaluateManageApiService;
import com.gic.evaluate.web.vo.UserReplyVO;
import com.gic.open.api.dto.EnterpriseServeInfoDTO;
import com.gic.open.api.service.EnterpriseLicenseApiService;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

/**
 * @author zhiwj
 * @Description:
 * @date 2019-12-30 18:33
 */
@RestController
public class EvaluateManagerController {


    @Autowired
    private EvaluateManageApiService evaluateManageApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private ResourceApiService resourceApiService;
    @Autowired
    EnterpriseLicenseApiService enterpriseLicenseApiService;


    @RequestMapping("/license")
    public RestResponse license() {
        ServiceResponse<EnterpriseServeInfoDTO> info = enterpriseLicenseApiService.getServeInfo(UserDetailUtils.getUserDetail().getEnterpriseId(), "DDPJ");
        return ResultControllerUtils.commonResult(info);
    }

    @RequestMapping("/list-evaluate")
    public RestResponse listEvaluate(EvaluateQO evaluateQO) {
        evaluateQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());

        // 判断权限
        OrderResourceDTO orderResourceDTO = resourceApiService.getOrderResourceByUserId(UserDetailUtils.getUserDetail().getUserId()).getResult();
        if (ResourceHoldEnum.DART.getCode().equals(orderResourceDTO.getType())) {
            if (CollectionUtils.isEmpty(orderResourceDTO.getStoreIdList())) {
                return RestResponse.success();
            } else {
                evaluateQO.setStoreIdList(orderResourceDTO.getStoreIdList());
            }
        }

        if (evaluateQO.getStoreWidgetId() != null) {
            ServiceResponse<List<Integer>> serviceResponse = storeWidgetApiService.listStoreIdByStoreWidgetId(UserDetailUtils.getUserDetail().getEnterpriseId(), evaluateQO.getStoreWidgetId());
            evaluateQO.setStoreIdList(serviceResponse.getResult());
        }

        if (StringUtils.isNotBlank(evaluateQO.getStartTime())) {
            evaluateQO.setStartTime(evaluateQO.getStartTime() + " 00:00:00");
        }
        if (StringUtils.isNotBlank(evaluateQO.getEndTime())) {
            evaluateQO.setEndTime(evaluateQO.getEndTime() + " 23:59:59");
        }

        if (Constants.CLOSE.equals(evaluateQO.getBadEvaluateStatus())) {
            evaluateQO.setBadEvaluateStatus(null);
        }
        if (Constants.CLOSE.equals(evaluateQO.getGoodEvaluateStatus())) {
            evaluateQO.setGoodEvaluateStatus(null);
        }
        if (Constants.CLOSE.equals(evaluateQO.getReviewStatus())) {
            evaluateQO.setReviewStatus(null);
        }
        if (Constants.CLOSE.equals(evaluateQO.getTopHotStatus())) {
            evaluateQO.setTopHotStatus(null);
        }
        if (Constants.CLOSE.equals(evaluateQO.getMaliciousStatus())) {
            evaluateQO.setMaliciousStatus(null);
        }
        ServiceResponse<Page<EvaluateDTO>> serviceResponse = evaluateManageApiService.listEvaluateForManage(evaluateQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-evaluate")
    public RestResponse getEvaluate(Integer evaluateId) {
        ServiceResponse<EvaluateDTO> serviceResponse = evaluateManageApiService.getEvaluate(UserDetailUtils.getUserDetail().getEnterpriseId(), evaluateId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-show-status")
    public RestResponse updateShowStatus(Integer[] evaluateIdList, Integer showStatus) {
        ServiceResponse<Void> serviceResponse = evaluateManageApiService.updateShowStatus(UserDetailUtils.getUserDetail().getEnterpriseId(), Arrays.asList(evaluateIdList), showStatus);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-reply-show-status")
    public RestResponse updateReplyShowStatus(Integer evaluateReplyId, Integer showStatus) {
        ServiceResponse<Void> serviceResponse = evaluateManageApiService.updateReplyShowStatus(UserDetailUtils.getUserDetail().getEnterpriseId(), evaluateReplyId, showStatus);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-top-hot-status")
    public RestResponse updateTopHotStatus(Integer evaluateId, Integer topHotStatus) {
        ServiceResponse<Void> serviceResponse = evaluateManageApiService.updateTopHotStatus(UserDetailUtils.getUserDetail().getEnterpriseId(), evaluateId, topHotStatus);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-malicious-status")
    public RestResponse updateMaliciousStatus(Integer evaluateId, Integer maliciousStatus) {
        ServiceResponse<Void> serviceResponse = evaluateManageApiService.updateMaliciousStatus(UserDetailUtils.getUserDetail().getEnterpriseId(), evaluateId, maliciousStatus);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/reply-evaluate")
    public RestResponse replyEvaluate(UserReplyVO userReplyVO) {
        userReplyVO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        userReplyVO.setUserId(UserDetailUtils.getUserDetail().getUserId());
        userReplyVO.setUserName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        ServiceResponse<Void> serviceResponse = evaluateManageApiService.replyEvaluate(EntityUtil.changeEntityByJSON(UserReplyDTO.class, userReplyVO));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/edit-reply-evaluate")
    public RestResponse editReplyEvaluate(UserReplyVO userReplyVO) {
        userReplyVO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        userReplyVO.setUserId(UserDetailUtils.getUserDetail().getUserId());
        userReplyVO.setUserName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        ServiceResponse<Void> serviceResponse = evaluateManageApiService.editReplyEvaluate(EntityUtil.changeEntityByJSON(UserReplyDTO.class, userReplyVO));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/evaluate-quick-reply")
    public RestResponse listEvaluateQuickReply(Integer quickReplyType) {
        ServiceResponse<List<EvaluateQuickDTO>> serviceResponse = evaluateManageApiService.listEvaluateQuickReply(UserDetailUtils.getUserDetail().getEnterpriseId(), quickReplyType);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/del-reply")
    public RestResponse delReply(Integer evaluateReplyId) {
        ServiceResponse<Void> serviceResponse = evaluateManageApiService.delReply(evaluateReplyId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

}
