package com.gic.evaluate.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.ResourceHoldEnum;
import com.gic.auth.dto.StoreResourceDTO;
import com.gic.auth.service.ResourceApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.evaluate.dto.ProblemDTO;
import com.gic.evaluate.dto.ProblemUserReplyDTO;
import com.gic.evaluate.qo.ProblemQO;
import com.gic.evaluate.service.ProblemManageApiService;
import com.gic.evaluate.web.vo.ProblemUserReplyVO;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-02 16:05
 */
@RestController
public class ProblemController {

    @Autowired
    private ProblemManageApiService problemManageApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private ResourceApiService resourceApiService;

    @RequestMapping("/list-problem")
    public RestResponse listProblem(ProblemQO problemQO) {
        problemQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        // 权限判断
        StoreResourceDTO storeResource = resourceApiService.getStoreResourceByUserId(UserDetailUtils.getUserDetail().getUserId()).getResult();
        if (ResourceHoldEnum.DART.getCode().equals(storeResource.getType())) {
            if (CollectionUtils.isEmpty(storeResource.getStoreIdList())) {
                return RestResponse.success();
            } else {
                problemQO.setStoreIdList(storeResource.getStoreIdList());
            }
        }

        if (problemQO.getStoreWidgetId() != null) {
            ServiceResponse<List<Integer>> serviceResponse = storeWidgetApiService.listStoreIdByStoreWidgetId(UserDetailUtils.getUserDetail().getEnterpriseId(), problemQO.getStoreWidgetId());
            problemQO.setStoreIdList(serviceResponse.getResult());
        }
        if (StringUtils.isNotBlank(problemQO.getStartTime())) {
            problemQO.setStartTime(problemQO.getStartTime() + " 00:00:00");
        }
        if (StringUtils.isNotBlank(problemQO.getEndTime())) {
            problemQO.setEndTime(problemQO.getEndTime() + " 23:59:59");
        }

        ServiceResponse<Page<ProblemDTO>> serviceResponse = problemManageApiService.listProblem(problemQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-problem")
    public RestResponse getProblem(Integer problemId) {
        ServiceResponse<ProblemDTO> serviceResponse = problemManageApiService.getProblem(problemId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/reply-problem")
    public RestResponse replyProblem(ProblemUserReplyVO problemUserReplyVO) {
        problemUserReplyVO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        problemUserReplyVO.setUserId(UserDetailUtils.getUserDetail().getUserId());
        problemUserReplyVO.setUserName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        ServiceResponse<Void> serviceResponse = problemManageApiService.replyProblem(Arrays.asList(problemUserReplyVO.getProblemIds()), EntityUtil.changeEntityByJSON(ProblemUserReplyDTO.class, problemUserReplyVO));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/edit-reply-problem")
    public RestResponse editReplyProblem(ProblemUserReplyVO problemUserReplyVO) {
        problemUserReplyVO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        problemUserReplyVO.setUserId(UserDetailUtils.getUserDetail().getUserId());
        problemUserReplyVO.setUserName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        ServiceResponse<Void> serviceResponse = problemManageApiService.editReplyProblem(EntityUtil.changeEntityByJSON(ProblemUserReplyDTO.class, problemUserReplyVO));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-problem-malicious-status")
    public RestResponse updateMaliciousStatus(Integer problemId, Integer maliciousStatus) {
        ServiceResponse<Void> serviceResponse = problemManageApiService.updateMaliciousStatus(problemId, maliciousStatus);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

}
