package com.gic.evaluate.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.DataDownloadUtils;
import com.gic.download.utils.DownloadReport;
import com.gic.download.utils.DownloadUtils;
import com.gic.download.utils.ExecutorPoolSingleton;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.evaluate.constant.ReplyerTypeEnum;
import com.gic.evaluate.dto.ProblemDTO;
import com.gic.evaluate.dto.ProblemReplyDTO;
import com.gic.evaluate.qo.ExportQO;
import com.gic.evaluate.qo.ProblemQO;
import com.gic.evaluate.service.ProblemManageApiService;
import com.gic.evaluate.web.utils.DataContentUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-14 16:47
 */
@RestController
public class ProblemExportController extends DownloadUtils {

    @Autowired
    private ProblemManageApiService problemManageApiService;

    private Logger logger = LogManager.getLogger(ProblemExportController.class);

    @RequestMapping("/list-problem-field")
    public RestResponse listProblemField() {
        ProblemFieldEnum[] values = ProblemFieldEnum.values();
        List<Map<String, Object>> arrayList = new ArrayList<>(values.length);
        for (ProblemFieldEnum value : values) {
            Map<String, Object> hashMap = new HashMap<>(4);
            hashMap.put("fieldCode", value.getFieldCode());
            hashMap.put("fieldName", value.getFieldName());
            hashMap.put("needEncrypt", value.getNeedEncrypt());
            arrayList.add(hashMap);
        }
        return RestResponse.success(arrayList);
    }

    @RequestMapping("/export-problem")
    public RestResponse exportProblemRecord(HttpServletRequest request, ProblemQO problemQO, ExportQO exportQO) {
        problemQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (problemQO.getProblemIds() != null && problemQO.getProblemIds().length == 1 && problemQO.getProblemIds()[0] == -1) {
            // 查询全部
            problemQO.setProblemIds(null);
        }
        ServiceResponse<Integer> serviceResponse = problemManageApiService.countProblem(problemQO);

        List<String> fieldCodeList = exportQO.getFieldCodeList();
        List<String> fieldNameList = new ArrayList<>(fieldCodeList.size());
        for (String fieldCode: fieldCodeList) {
            String fieldName = ProblemFieldEnum.valueOf(fieldCode).getFieldName();
            fieldNameList.add(fieldName);
        }

        // 开始下载
        String dataContent = DataContentUtils.getDataContent();

        // 生成入参数据
        DownloadReport report = new DownloadReport();
        //申请人ID
        report.setApplyUserId(UserDetailUtils.getUserDetail().getUserId());
        //数据源（频道页），用  "-" 隔开上级和本级
        report.setDataContent(dataContent);
        //待下载数据的数据总量
        report.setDataCount(serviceResponse.getResult());
        //报告类型  1：完整数据， 2：脱敏数据， 3：二维码
        report.setDataType(exportQO.getDataType());
        //数据源超链接，用于点击跳转对应频道页面
        report.setDataUrl("-");
        report.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //待下载文件名
        report.setFileName(exportQO.getFileName());
        //待下载数据的结束时间
        if (StringUtils.isNotBlank(problemQO.getEndTime())) {
            report.setReportEndTime(DateUtil.strToDate(problemQO.getEndTime(), DateUtil.FORMAT_DATETIME_19));
        }
        //待下载数据的开始时间
        if (StringUtils.isNotBlank(problemQO.getStartTime())) {
            report.setReportStartTime(DateUtil.strToDate(problemQO.getStartTime(), DateUtil.FORMAT_DATETIME_19));
        }
        //调用生成方法，获取reportId
        Integer reportId = DataDownloadUtils.createDownloadReport(report);

        String path = request.getSession().getServletContext().getRealPath("/excel/csv/collaborator/file/");
        ExecutorPoolSingleton.getInstance().executeTask(() -> {
            DownloadDataLoader<ProblemDTO> loader = new DownloadDataLoader<ProblemDTO>() {
                @Override
                protected List<ProblemDTO> getDownloadData(Integer pageNum) throws Exception {
                    //调用分页查询接口
                    problemQO.setCurrentPage(pageNum);
                    problemQO.setPageSize(1000);
                    ServiceResponse<Page<ProblemDTO>> result = problemManageApiService.listProblem(problemQO);
                    boolean hasReplyUserContent = fieldCodeList.contains("replyUserContent");
                    List<ProblemDTO> resultList = result.getResult().getResult();
                    if (CollectionUtils.isNotEmpty(resultList)) {
                        for(ProblemDTO resultDTO : resultList) {
                            if (hasReplyUserContent) {
                                ProblemDTO problemDTO = problemManageApiService.getProblem(resultDTO.getProblemId()).getResult();

                                List<ProblemReplyDTO> replyList = problemDTO.getReplyList();
//                                List<String> replyUserContentList = Optional.ofNullable(replyList).orElse(Collections.emptyList()).stream()
//                                        .filter(e -> ReplyerTypeEnum.USER.getCode().equals(e.getReplyerType()))
//                                        .map(ProblemReplyDTO::getReplyContent)
//                                        .collect(Collectors.toList());
                                int j = 0;
                                StringBuilder sb = new StringBuilder();
                                for (ProblemReplyDTO problemReplyDTO : replyList) {
                                    if (ReplyerTypeEnum.MEMBER.getCode().equals(problemReplyDTO.getReplyerType())) {
                                        j++;
                                        sb.append("(").append(j).append(")").append(problemReplyDTO.getReplyContent()).append("\r\n");
                                    }
                                }
                                resultDTO.setReplyUserContent(sb.toString());

                            }
                        }
                    }

                    return resultList;
                }
            };
            try {

                /*
                //参数说明：excelExtensionCode，Excel后缀名code ，枚举类：ExcelExtensionEnum,
                // Arrays.asList("phone")这个phone是需要加密的字段
                 */
                download(path, reportId, exportQO.getFileName(), exportQO.getExcelExtension(), fieldNameList, fieldCodeList, loader, Arrays.asList("memberPhone"), null);
            } catch (Exception e) {
                logger.warn("下载错误", e);
            }
        });
        return RestResponse.success(reportId);
    }

    private enum ProblemFieldEnum {
        /**
         * 首次评价时间
         */
        createTime("createTime", "首次评价时间", 0),
        /**
         * 会员姓名
         */
        memberName("memberName", "会员姓名", 0),
        /**
         * 会员手机号
         */
        memberPhone("memberPhone", "会员手机号", 1),
        /**
         * 内容概览
         */
        problemContent("problemContent", "内容概览", 0),
        /**
         * 门店名称
         */
        storeName("storeName", "门店名称", 0),
        /**
         * 商家回复
         */
        replyUserContent("replyUserContent", "商家回复", 0),
        ;
        private String fieldCode;
        private String fieldName;
        private Integer needEncrypt;

        ProblemFieldEnum(String fieldCode, String fieldName, Integer needEncrypt) {
            this.fieldCode = fieldCode;
            this.fieldName = fieldName;
            this.needEncrypt = needEncrypt;
        }

        public String getFieldCode() {
            return fieldCode;
        }

        public String getFieldName() {
            return fieldName;
        }

        public Integer getNeedEncrypt() {
            return needEncrypt;
        }
    }

}